# coding=utf-8

import os
import platform
import unittest

import pytest
from parameterized import parameterized

from conans import MSBuild, tools
from conans.client.runner import ConanRunner
from conans.test.utils.mocks import MockSettings, MockConanfile, TestBufferConanOutput
from conans.test.utils.tools import TestClient
from conans.test.assets.visual_project_files import get_vs_project_files

main_cpp = r"""#include <hello.h>

int main(){
    hello();
}
"""

conanfile_txt = r"""[requires]
Hello1/0.1@lasote/testing
[generators]
{generator}
"""

hello_conanfile_py = r"""from conans import ConanFile

class HelloConan(ConanFile):
    name = "Hello1"
    version = "0.1"
    exports = '*'

    def package(self):
        self.copy("*.h", dst="include")

    def package_info(self):
        self.cpp_info.debug.defines = ["CONAN_DEBUG"]
        self.cpp_info.release.defines = ["CONAN_RELEASE"]
"""


hello_h = r"""#include <iostream>

#ifdef CONAN_DEBUG
void hello(){
    std::cout << "Hello Debug!!!" << std::endl;
}
#endif
#ifdef CONAN_RELEASE
void hello(){
    std::cout << "Hello Release!!!" << std::endl;
}
#endif
"""


@pytest.mark.slow
@pytest.mark.tool_visual_studio
@pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
class VisualStudioMultiTest(unittest.TestCase):

    @parameterized.expand([("visual_studio", "conanbuildinfo.props"),
                           ("visual_studio_multi", "conanbuildinfo_multi.props")])
    def test_build_vs_project(self, generator, props):
        client = TestClient()
        client.save({"conanfile.py": hello_conanfile_py,
                     "hello.h": hello_h})
        client.run("create . lasote/testing")

        files = get_vs_project_files()
        files["MyProject/main.cpp"] = main_cpp
        files["conanfile.txt"] = conanfile_txt.format(generator=generator)
        props = os.path.join(client.current_folder, props)
        old = r'<Import Project="$(VCTargetsPath)\Microsoft.Cpp.targets" />'
        new = old + '<Import Project="{props}" />'.format(props=props)
        files["MyProject/MyProject.vcxproj"] = files["MyProject/MyProject.vcxproj"].replace(old, new)
        client.save(files, clean_first=True)

        for build_type in ["Debug", "Release"]:
            arch = "x86"
            runner = ConanRunner(print_commands_to_output=True,
                                 generate_run_log_file=False,
                                 log_run_to_output=True,
                                 output=TestBufferConanOutput())
            settings = MockSettings({"os": "Windows",
                                     "build_type": build_type,
                                     "arch": arch,
                                     "compiler": "Visual Studio",
                                     "compiler.version": "15",
                                     "compiler.toolset": "v141"})
            conanfile = MockConanfile(settings, runner=runner)
            settings = " -s os=Windows " \
                       " -s build_type={build_type} " \
                       " -s arch={arch}" \
                       " -s compiler=\"Visual Studio\"" \
                       " -s compiler.toolset=v141" \
                       " -s compiler.version=15".format(build_type=build_type, arch=arch)
            client.run("install . %s" % settings)
            with tools.chdir(client.current_folder):
                msbuild = MSBuild(conanfile)
                msbuild.build(project_file="MyProject.sln", build_type=build_type, arch=arch)
                output = TestBufferConanOutput()
                client.run_command(r"%s\MyProject.exe" % build_type)
                self.assertIn("Hello %s!!!" % build_type, client.out)



import textwrap

from conans.client.graph.graph import CONTEXT_HOST, CONTEXT_BUILD
from conans.model.profile import Profile
from conans.test.integration.graph.core.cross_build.protoc_basic_test import ClassicProtocExampleBase


class ProtocWithGTestExample(ClassicProtocExampleBase):
    """ Built on top of the ClassicProtocExample, in this use case we are adding a testing library
        to the project: we add gtest as a build_require and also the protoc executable, BUT
        both of them should be compiled for the host platform, we are running tests in the host!
    """

    application = textwrap.dedent("""
        from conans import ConanFile

        class Protoc(ConanFile):
            name = "application"
            version = "testing"

            settings = "os"

            def requirements(self):
                self.requires("protobuf/testing@user/channel")

            def build_requirements(self):
                self.build_requires("protoc/testing@user/channel", force_host_context=False)
                # Make it explicit that these should be for host_machine
                self.build_requires("protoc/testing@user/channel", force_host_context=True)
                self.build_requires("gtest/testing@user/channel", force_host_context=True)

            def build(self):
                self.output.info(">> settings.os:".format(self.settings.os))
    """)

    def setUp(self):
        super(ProtocWithGTestExample, self).setUp()
        self._cache_recipe(self.gtest_ref, self.gtest)
        self._cache_recipe(self.app_ref, self.application)

    def test_not_crossbuilding(self):
        """ Build_requires uses 'force_host_context=False', but no profile for build_machine """
        profile_host = Profile()
        profile_host.settings["os"] = "Host"
        profile_host.process_settings(self.cache)

        try:
            deps_graph = self._build_graph(profile_host=profile_host, profile_build=None, install=True)
        except Exception as e:
            self.fail("ERROR! Although the recipe specifies 'force_host_context=False', as we"
                      " are not providing a profile for the build_machine, the context will be"
                      " host for this build_require.\n Exception has been: {}".format(e))
        else:
            # Check packages (all are HOST packages)
            application = deps_graph.root.dependencies[0].dst
            self.assertEqual(len(application.dependencies), 3)
            self.assertEqual(application.conanfile.name, "app")
            self.assertEqual(application.context, CONTEXT_HOST)
            self.assertEqual(application.conanfile.settings.os, profile_host.settings['os'])

            protobuf_host = application.dependencies[0].dst
            self.assertEqual(protobuf_host.conanfile.name, "protobuf")
            self.assertEqual(protobuf_host.context, CONTEXT_HOST)
            self.assertEqual(protobuf_host.conanfile.settings.os, profile_host.settings['os'])

            protoc_host = application.dependencies[1].dst
            self.assertEqual(protoc_host.conanfile.name, "protoc")
            self.assertEqual(protoc_host.context, CONTEXT_HOST)
            self.assertEqual(protoc_host.conanfile.settings.os, profile_host.settings['os'])

            protoc_protobuf_host = protoc_host.dependencies[0].dst
            self.assertEqual(protoc_protobuf_host, protobuf_host)

            gtest_host = application.dependencies[2].dst
            self.assertEqual(gtest_host.conanfile.name, "gtest")
            self.assertEqual(gtest_host.context, CONTEXT_HOST)
            self.assertEqual(gtest_host.conanfile.settings.os, profile_host.settings['os'])

    def test_crossbuilding(self):
        profile_host = Profile()
        profile_host.settings["os"] = "Host"
        profile_host.process_settings(self.cache)

        profile_build = Profile()
        profile_build.settings["os"] = "Build"
        profile_build.process_settings(self.cache)

        deps_graph = self._build_graph(profile_host=profile_host, profile_build=profile_build, install=True)

        # Check HOST packages
        #   - Application
        application = deps_graph.root.dependencies[0].dst
        self.assertEqual(len(application.dependencies), 4)
        self.assertEqual(application.conanfile.name, "app")
        self.assertEqual(application.context, CONTEXT_HOST)
        self.assertEqual(application.conanfile.settings.os, profile_host.settings['os'])

        #   - Application::deps_cpp_info:
        protobuf_cpp_info = application.conanfile.deps_cpp_info["protobuf"]
        self.assertEqual(protobuf_cpp_info.includedirs, ['protobuf-host'])
        self.assertEqual(protobuf_cpp_info.libdirs, ['protobuf-host'])
        self.assertEqual(protobuf_cpp_info.bindirs, ['protobuf-host'])

        protoc_cpp_info = application.conanfile.deps_cpp_info["protoc"]
        self.assertEqual(protoc_cpp_info.includedirs, ['protoc-host'])
        self.assertEqual(protoc_cpp_info.libdirs, ['protoc-host'])
        self.assertEqual(protoc_cpp_info.bindirs, ['protoc-host'])

        gtest_cpp_info = application.conanfile.deps_cpp_info["gtest"]
        self.assertEqual(gtest_cpp_info.includedirs, ['gtest-host'])
        self.assertEqual(gtest_cpp_info.libdirs, ['gtest-host'])
        self.assertEqual(gtest_cpp_info.bindirs, ['gtest-host'])

        #   - Application::deps_env_info:
        protoc_env_info = application.conanfile.deps_env_info["protoc"]
        self.assertEqual(protoc_env_info.PATH, ['protoc-build'])
        self.assertEqual(protoc_env_info.OTHERVAR, 'protoc-build')

        protobuf_env_info = application.conanfile.deps_env_info["protobuf"]
        self.assertEqual(protobuf_env_info.PATH, ['protobuf-build'])
        self.assertEqual(protobuf_env_info.OTHERVAR, 'protobuf-build')

        with self.assertRaises(KeyError):
            _ = application.conanfile.deps_env_info["gtest"]

        #   - Protobuf
        protobuf_host = application.dependencies[0].dst
        self.assertEqual(protobuf_host.conanfile.name, "protobuf")
        self.assertEqual(protobuf_host.context, CONTEXT_HOST)
        self.assertEqual(protobuf_host.conanfile.settings.os, profile_host.settings['os'])

        #   - Protoc
        protoc_host = application.dependencies[2].dst
        self.assertEqual(protoc_host.conanfile.name, "protoc")
        self.assertEqual(protoc_host.context, CONTEXT_HOST)
        self.assertEqual(protoc_host.conanfile.settings.os, profile_host.settings['os'])

        #   - Protobuf <- Protoc
        protoc_protobuf_host = protoc_host.dependencies[0].dst
        self.assertEqual(protoc_protobuf_host, protobuf_host)

        #   - GTest
        gtest_host = application.dependencies[3].dst
        self.assertEqual(gtest_host.conanfile.name, "gtest")
        self.assertEqual(gtest_host.context, CONTEXT_HOST)
        self.assertEqual(gtest_host.conanfile.settings.os, profile_host.settings['os'])

        # Check BUILD packages (default behavior changes if we use profile_build)
        #   - Protoc
        protoc_build = application.dependencies[1].dst
        self.assertEqual(protoc_build.conanfile.name, "protoc")
        self.assertEqual(protoc_build.context, CONTEXT_BUILD)
        self.assertEqual(str(protoc_build.conanfile.settings.os), profile_build.settings['os'])

        #   - Protobuf
        protobuf_build = protoc_build.dependencies[0].dst
        self.assertEqual(protobuf_build.conanfile.name, "protobuf")
        self.assertEqual(protoc_build.context, CONTEXT_BUILD)
        self.assertEqual(str(protobuf_build.conanfile.settings.os), profile_build.settings['os'])



import os
import re
import sys

import pytest

from conans.model.ref import ConanFileReference, PackageReference
from conans.test.utils.tools import TestClient


@pytest.mark.tool_meson
@pytest.mark.skipif(sys.version_info.major == 2, reason="Meson not supported in Py2")
@pytest.mark.tool_pkg_config
def test_meson_lib_template():
    # Identical to def test_cmake_lib_template(), but for Meson
    client = TestClient(path_with_spaces=False)
    client.run("new hello/0.1 --template=meson_lib")

    # Local flow works
    client.run("install . -if=install")
    client.run("build . -if=install")
    client.run("export-pkg . hello/0.1@ -if=install")
    package_id = re.search(r"Packaging to (\S+)", str(client.out)).group(1)
    pref = PackageReference(ConanFileReference.loads("hello/0.1"), package_id)
    package_folder = client.cache.package_layout(pref.ref).package(pref)
    assert os.path.exists(os.path.join(package_folder, "include", "hello.h"))

    # Create works
    client.run("create .")
    assert "hello/0.1: Hello World Release!" in client.out

    client.run("create . -s build_type=Debug")
    assert "hello/0.1: Hello World Debug!" in client.out

    # Create + shared works
    client.run("create . -o hello:shared=True")
    assert "hello/0.1: Hello World Release!" in client.out


@pytest.mark.tool_meson
@pytest.mark.skipif(sys.version_info.major == 2, reason="Meson not supported in Py2")
def test_meson_exe_template():
    client = TestClient(path_with_spaces=False)
    client.run("new greet/0.1 --template=meson_exe")
    # Local flow works
    client.run("install . -if=install")
    client.run("build . -if=install")

    # Create works
    client.run("create .")
    assert "greet/0.1: Hello World Release!" in client.out

    client.run("create . -s build_type=Debug")
    assert "greet/0.1: Hello World Debug!" in client.out



import textwrap
import unittest

from conans.test.utils.tools import TestClient


class TestRecipeAttribute(unittest.TestCase):

    def test_invalid_description(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
                from conans import ConanFile
                class Pkg(ConanFile):
                    description = ("foo", "bar")
                """)
        client.save({"conanfile.py": conanfile})
        client.run("create . pkg/0.1@user/testing", assert_error=True)

        self.assertIn("Recipe 'description' must be a string.", client.out)



import os
import re
import unittest

import six
from mock import patch, Mock

import conans
from conans.client.build.cmake import CMake
from conans.client.build.cmake_flags import CMakeDefinitionsBuilder
from conans.client.conf import get_default_settings_yml
from conans.client.generators import CMakeFindPackageGenerator, CMakeFindPackageMultiGenerator
from conans.client.generators.cmake import CMakeGenerator
from conans.client.generators.cmake_multi import CMakeMultiGenerator
from conans.errors import ConanException
from conans.model.build_info import CppInfo
from conans.model.conan_file import ConanFile
from conans.model.env_info import EnvValues
from conans.model.ref import ConanFileReference
from conans.model.settings import Settings
from conans.test.utils.test_files import temp_folder
from conans.util.files import save


class _MockSettings(object):
    build_type = None
    os = None
    os_build = None
    fields = []

    def __init__(self, build_type=None):
        class BuildType(str):
            values_range = ["Debug", "Release"]
        self.build_type = BuildType(build_type)

    @property
    def compiler(self):
        raise ConanException("mock: not available")

    def constraint(self, _):
        return self

    def get_safe(self, name):
        if name == "build_type":
            return self.build_type
        if name == "os":
            return self.os
        if name == "os_build":
            return self.os_build
        return None

    def items(self):
        return {}


class CMakeGeneratorTest(unittest.TestCase):

    @staticmethod
    def _extract_macro(name, text):
        pattern = r".*(macro\(%s\).*?endmacro\(\)).*" % name
        return re.sub(pattern, r"\1", text, flags=re.DOTALL)

    def test_variables_setup(self):
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        ref = ConanFileReference.loads("MyPkg/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder1")
        cpp_info.defines = ["MYDEFINE1"]
        conanfile.deps_cpp_info.add(ref.name, cpp_info)
        ref = ConanFileReference.loads("MyPkg2/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder2")
        cpp_info.defines = ["MYDEFINE2"]
        conanfile.deps_cpp_info.add(ref.name, cpp_info)
        conanfile.deps_user_info["LIB1"].myvar = "myvalue"
        conanfile.deps_user_info["LIB1"].myvar2 = "myvalue2"
        conanfile.deps_user_info["lib2"].MYVAR2 = "myvalue4"
        generator = CMakeGenerator(conanfile)
        content = generator.content
        cmake_lines = content.splitlines()
        self.assertIn('set(CONAN_DEFINES_MYPKG "-DMYDEFINE1")', cmake_lines)
        self.assertIn('set(CONAN_DEFINES_MYPKG2 "-DMYDEFINE2")', cmake_lines)
        self.assertIn('set(CONAN_COMPILE_DEFINITIONS_MYPKG "MYDEFINE1")', cmake_lines)
        self.assertIn('set(CONAN_COMPILE_DEFINITIONS_MYPKG2 "MYDEFINE2")', cmake_lines)

        self.assertIn('set(CONAN_USER_LIB1_myvar "myvalue")', cmake_lines)
        self.assertIn('set(CONAN_USER_LIB1_myvar2 "myvalue2")', cmake_lines)
        self.assertIn('set(CONAN_USER_LIB2_MYVAR2 "myvalue4")', cmake_lines)

    def test_paths_cmake_multi_user_vars(self):
        settings_mock = _MockSettings(build_type="Release")
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(settings_mock, EnvValues())
        ref = ConanFileReference.loads("MyPkg/0.1@lasote/stables")
        tmp_folder = temp_folder()
        save(os.path.join(tmp_folder, "lib", "mylib.lib"), "")
        save(os.path.join(tmp_folder, "include", "myheader.h"), "")
        cpp_info = CppInfo(ref.name, tmp_folder)
        cpp_info.release.libs = ["hello"]
        cpp_info.debug.libs = ["hello_D"]
        conanfile.deps_cpp_info.add(ref.name, cpp_info)
        generator = CMakeMultiGenerator(conanfile)
        release = generator.content["conanbuildinfo_release.cmake"]
        release = release.replace(tmp_folder.replace("\\", "/"), "root_folder")
        cmake_lines = release.splitlines()
        self.assertIn('set(CONAN_INCLUDE_DIRS_MYPKG_RELEASE "root_folder/include")', cmake_lines)
        self.assertIn('set(CONAN_LIB_DIRS_MYPKG_RELEASE "root_folder/lib")', cmake_lines)

    def test_paths_cmake(self):
        settings_mock = _MockSettings()
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(settings_mock, EnvValues())
        ref = ConanFileReference.loads("MyPkg/0.1@lasote/stables")
        tmp_folder = temp_folder()
        save(os.path.join(tmp_folder, "lib", "mylib.lib"), "")
        save(os.path.join(tmp_folder, "include", "myheader.h"), "")
        cpp_info = CppInfo(ref.name, tmp_folder)
        cpp_info.release.libs = ["hello"]
        cpp_info.debug.libs = ["hello_D"]
        conanfile.deps_cpp_info.add(ref.name, cpp_info)
        generator = CMakeGenerator(conanfile)
        content = generator.content
        content = content.replace(tmp_folder.replace("\\", "/"), "root_folder")
        cmake_lines = content.splitlines()
        self.assertIn('set(CONAN_INCLUDE_DIRS_MYPKG_RELEASE "root_folder/include")', cmake_lines)
        self.assertIn('set(CONAN_LIB_DIRS_MYPKG_RELEASE "root_folder/lib")', cmake_lines)

    def test_variables_cmake_multi_user_vars(self):
        settings_mock = _MockSettings(build_type="Release")
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(settings_mock, EnvValues())
        conanfile.deps_user_info["LIB1"].myvar = "myvalue"
        conanfile.deps_user_info["LIB1"].myvar2 = "myvalue2"
        conanfile.deps_user_info["lib2"].MYVAR2 = "myvalue4"
        generator = CMakeMultiGenerator(conanfile)
        content = generator.content["conanbuildinfo_multi.cmake"]
        cmake_lines = content.splitlines()
        self.assertIn('set(CONAN_USER_LIB1_myvar "myvalue")', cmake_lines)
        self.assertIn('set(CONAN_USER_LIB1_myvar2 "myvalue2")', cmake_lines)
        self.assertIn('set(CONAN_USER_LIB2_MYVAR2 "myvalue4")', cmake_lines)

    def test_variables_cmake_multi_user_vars_escape(self):
        settings_mock = _MockSettings(build_type="Release")
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(settings_mock, EnvValues())
        conanfile.deps_user_info["FOO"].myvar = 'my"value"'
        conanfile.deps_user_info["FOO"].myvar2 = 'my${value}'
        conanfile.deps_user_info["FOO"].myvar3 = 'my\\value'
        generator = CMakeMultiGenerator(conanfile)
        content = generator.content["conanbuildinfo_multi.cmake"]
        cmake_lines = content.splitlines()
        self.assertIn(r'set(CONAN_USER_FOO_myvar "my\"value\"")', cmake_lines)
        self.assertIn(r'set(CONAN_USER_FOO_myvar2 "my\${value}")', cmake_lines)
        self.assertIn(r'set(CONAN_USER_FOO_myvar3 "my\\value")', cmake_lines)

    def test_multi_flag(self):
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        ref = ConanFileReference.loads("MyPkg/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder1")
        cpp_info.includedirs.append("other_include_dir")
        cpp_info.cxxflags = ["-DGTEST_USE_OWN_TR1_TUPLE=1", "-DGTEST_LINKED_AS_SHARED_LIBRARY=1"]
        conanfile.deps_cpp_info.add(ref.name, cpp_info)
        ref = ConanFileReference.loads("MyPkg2/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder2")
        cpp_info.cflags = ["-DSOMEFLAG=1"]
        conanfile.deps_cpp_info.add(ref.name, cpp_info)
        generator = CMakeGenerator(conanfile)
        content = generator.content
        cmake_lines = content.splitlines()
        self.assertIn('set(CONAN_C_FLAGS_MYPKG2 "-DSOMEFLAG=1")', cmake_lines)
        self.assertIn('set(CONAN_CXX_FLAGS_MYPKG "-DGTEST_USE_OWN_TR1_TUPLE=1'
                      ' -DGTEST_LINKED_AS_SHARED_LIBRARY=1")', cmake_lines)
        self.assertIn('set(CONAN_C_FLAGS "-DSOMEFLAG=1 ${CONAN_C_FLAGS}")', cmake_lines)
        self.assertIn('set(CONAN_CXX_FLAGS "-DGTEST_USE_OWN_TR1_TUPLE=1'
                      ' -DGTEST_LINKED_AS_SHARED_LIBRARY=1 ${CONAN_CXX_FLAGS}")', cmake_lines)

    def test_escaped_flags(self):
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        ref = ConanFileReference.loads("MyPkg/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder1")
        cpp_info.includedirs.append("other_include_dir")
        cpp_info.cxxflags = ["-load", r"C:\foo\bar.dll"]
        cpp_info.cflags = ["-load", r"C:\foo\bar2.dll"]
        cpp_info.defines = ['MY_DEF=My string', 'MY_DEF2=My other string']
        conanfile.deps_cpp_info.add(ref.name, cpp_info)
        generator = CMakeGenerator(conanfile)
        content = generator.content
        cmake_lines = content.splitlines()
        self.assertIn(r'set(CONAN_C_FLAGS_MYPKG "-load C:\\foo\\bar2.dll")', cmake_lines)
        self.assertIn(r'set(CONAN_CXX_FLAGS_MYPKG "-load C:\\foo\\bar.dll")', cmake_lines)
        self.assertIn(r'set(CONAN_DEFINES_MYPKG "-DMY_DEF=My string"', cmake_lines)
        self.assertIn('\t\t\t"-DMY_DEF2=My other string")', cmake_lines)

    def test_aux_cmake_test_setup(self):
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        generator = CMakeGenerator(conanfile)
        aux_cmake_test_setup = generator.content

        # extract the conan_basic_setup macro
        macro = self._extract_macro("conan_basic_setup", aux_cmake_test_setup)
        self.assertEqual("""macro(conan_basic_setup)
    set(options TARGETS NO_OUTPUT_DIRS SKIP_RPATH KEEP_RPATHS SKIP_STD SKIP_FPIC)
    cmake_parse_arguments(ARGUMENTS "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN} )

    if(CONAN_EXPORTED)
        conan_message(STATUS "Conan: called by CMake conan helper")
    endif()

    if(CONAN_IN_LOCAL_CACHE)
        conan_message(STATUS "Conan: called inside local cache")
    endif()

    if(NOT ARGUMENTS_NO_OUTPUT_DIRS)
        conan_message(STATUS "Conan: Adjusting output directories")
        conan_output_dirs_setup()
    endif()

    if(NOT ARGUMENTS_TARGETS)
        conan_message(STATUS "Conan: Using cmake global configuration")
        conan_global_flags()
    else()
        conan_message(STATUS "Conan: Using cmake targets configuration")
        conan_define_targets()
    endif()

    if(ARGUMENTS_SKIP_RPATH)
        # Change by "DEPRECATION" or "SEND_ERROR" when we are ready
        conan_message(WARNING "Conan: SKIP_RPATH is deprecated, it has been renamed to KEEP_RPATHS")
    endif()

    if(NOT ARGUMENTS_SKIP_RPATH AND NOT ARGUMENTS_KEEP_RPATHS)
        # Parameter has renamed, but we keep the compatibility with old SKIP_RPATH
        conan_set_rpath()
    endif()

    if(NOT ARGUMENTS_SKIP_STD)
        conan_set_std()
    endif()

    if(NOT ARGUMENTS_SKIP_FPIC)
        conan_set_fpic()
    endif()

    conan_check_compiler()
    conan_set_libcxx()
    conan_set_vs_runtime()
    conan_set_find_paths()
    conan_include_build_modules()
    conan_set_find_library_paths()
endmacro()""", macro)

        # extract the conan_set_find_paths macro
        macro = self._extract_macro("conan_set_find_paths", aux_cmake_test_setup)
        self.assertEqual("""macro(conan_set_find_paths)
    # CMAKE_MODULE_PATH does not have Debug/Release config, but there are variables
    # CONAN_CMAKE_MODULE_PATH_DEBUG to be used by the consumer
    # CMake can find findXXX.cmake files in the root of packages
    set(CMAKE_MODULE_PATH ${CONAN_CMAKE_MODULE_PATH} ${CMAKE_MODULE_PATH})

    # Make find_package() to work
    set(CMAKE_PREFIX_PATH ${CONAN_CMAKE_MODULE_PATH} ${CMAKE_PREFIX_PATH})

    # Set the find root path (cross build)
    set(CMAKE_FIND_ROOT_PATH ${CONAN_CMAKE_FIND_ROOT_PATH} ${CMAKE_FIND_ROOT_PATH})
    if(CONAN_CMAKE_FIND_ROOT_PATH_MODE_PROGRAM)
        set(CMAKE_FIND_ROOT_PATH_MODE_PROGRAM ${CONAN_CMAKE_FIND_ROOT_PATH_MODE_PROGRAM})
    endif()
    if(CONAN_CMAKE_FIND_ROOT_PATH_MODE_LIBRARY)
        set(CMAKE_FIND_ROOT_PATH_MODE_LIBRARY ${CONAN_CMAKE_FIND_ROOT_PATH_MODE_LIBRARY})
    endif()
    if(CONAN_CMAKE_FIND_ROOT_PATH_MODE_INCLUDE)
        set(CMAKE_FIND_ROOT_PATH_MODE_INCLUDE ${CONAN_CMAKE_FIND_ROOT_PATH_MODE_INCLUDE})
    endif()
endmacro()""", macro)

    def test_name_and_version_are_generated(self):
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        conanfile.name = "MyPkg"
        conanfile.version = "1.1.0"
        generator = CMakeGenerator(conanfile)
        content = generator.content
        cmake_lines = content.splitlines()
        self.assertIn('set(CONAN_PACKAGE_NAME MyPkg)', cmake_lines)
        self.assertIn('set(CONAN_PACKAGE_VERSION 1.1.0)', cmake_lines)

    def test_settings_are_generated(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "12"
        settings.compiler.runtime = "MD"
        settings.arch = "x86"
        settings.build_type = "Debug"
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        conanfile.settings = settings
        generator = CMakeGenerator(conanfile)
        content = generator.content
        cmake_lines = content.splitlines()
        self.assertIn('set(CONAN_SETTINGS_BUILD_TYPE "Debug")', cmake_lines)
        self.assertIn('set(CONAN_SETTINGS_ARCH "x86")', cmake_lines)
        self.assertIn('set(CONAN_SETTINGS_COMPILER "Visual Studio")', cmake_lines)
        self.assertIn('set(CONAN_SETTINGS_COMPILER_VERSION "12")', cmake_lines)
        self.assertIn('set(CONAN_SETTINGS_COMPILER_RUNTIME "MD")', cmake_lines)
        self.assertIn('set(CONAN_SETTINGS_OS "Windows")', cmake_lines)

    def test_cmake_find_package_multi_definitions(self):
        # CMAKE_PREFIX_PATH and CMAKE_MODULE_PATH must be in cmake_find_package_multi definitions

        settings_mock = _MockSettings(build_type="Release")
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(settings_mock, EnvValues())
        install_folder = "/c/foo/testing"
        conanfile.folders.set_base_install(install_folder)
        conanfile.generators = ["cmake_find_package_multi"]
        definitions_builder = CMakeDefinitionsBuilder(conanfile)
        definitions = definitions_builder.get_definitions("3.13")
        self.assertEqual(install_folder, definitions["CMAKE_PREFIX_PATH"])
        self.assertEqual(install_folder, definitions["CMAKE_MODULE_PATH"])

    def test_cmake_definitions_apple(self):
        # https://github.com/conan-io/conan/issues/7875
        settings_mock = _MockSettings(build_type="Release")
        settings_mock.os = "iOS"
        settings_mock.os_build = "Macos"
        conanfile = ConanFile(Mock(), None)
        conanfile.folders.set_base_install("/c/foo/testing")
        conanfile.initialize(settings_mock, EnvValues())
        definitions_builder = CMakeDefinitionsBuilder(conanfile)
        definitions = definitions_builder.get_definitions("3.13")
        self.assertEqual("Darwin", definitions["CMAKE_SYSTEM_NAME"])
        definitions = definitions_builder.get_definitions("3.14")
        self.assertEqual("iOS", definitions["CMAKE_SYSTEM_NAME"])
        definitions = definitions_builder.get_definitions(None)
        self.assertEqual("Darwin", definitions["CMAKE_SYSTEM_NAME"])

    def test_cmake_definitions_cmake_not_in_path(self):
        def raise_get_version():
            raise ConanException('Error retrieving CMake version')

        with patch.object(conans.client.build.cmake.CMake, "get_version",
                          side_effect=raise_get_version):
            settings_mock = _MockSettings(build_type="Release")
            conanfile = ConanFile(Mock(), None)
            install_folder = "/c/foo/testing"
            conanfile.folders.set_base_install(install_folder)
            conanfile.initialize(settings_mock, EnvValues())
            assert CMake(conanfile)

    def test_apple_frameworks(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Macos"
        settings.compiler = "apple-clang"
        settings.compiler.version = "9.1"
        settings.compiler.libcxx = "libc++"
        settings.arch = "x86_64"
        settings.build_type = "Debug"
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        conanfile.settings = settings

        ref = ConanFileReference.loads("MyPkg/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder1")
        cpp_info.frameworkdirs.extend(["path/to/Frameworks1", "path/to/Frameworks2"])
        cpp_info.frameworks = ["OpenGL", "OpenCL"]
        cpp_info.filter_empty = False
        conanfile.deps_cpp_info.add(ref.name, cpp_info)

        generator = CMakeGenerator(conanfile)
        content = generator.content
        self.assertIn('find_library(CONAN_FRAMEWORK_${_FRAMEWORK}_FOUND NAMES ${_FRAMEWORK} PATHS'
                      ' ${CONAN_FRAMEWORK_DIRS${SUFFIX}} CMAKE_FIND_ROOT_PATH_BOTH)', content)
        self.assertIn('set(CONAN_FRAMEWORK_DIRS "dummy_root_folder1/Frameworks"\n'
                      '\t\t\t"dummy_root_folder1/path/to/Frameworks1"\n'
                      '\t\t\t"dummy_root_folder1/path/to/Frameworks2" '
                      '${CONAN_FRAMEWORK_DIRS})', content)
        self.assertIn('set(CONAN_LIBS ${CONAN_LIBS} ${CONAN_SYSTEM_LIBS} '
                      '${CONAN_FRAMEWORKS_FOUND})', content)

        generator = CMakeFindPackageGenerator(conanfile)
        content = generator.content
        content = content['FindMyPkg.cmake']
        self.assertIn('conan_find_apple_frameworks(MyPkg_FRAMEWORKS_FOUND "${MyPkg_FRAMEWORKS}"'
                      ' "${MyPkg_FRAMEWORK_DIRS}")', content)


class CMakeCppInfoNameTest(unittest.TestCase):
    """
    Test cpp_info.name values are applied in generators instead of the reference name
    """

    def setUp(self):
        self.conanfile = ConanFile(Mock(), None)
        settings = _MockSettings(build_type="Debug")
        self.conanfile.initialize(settings, EnvValues())
        ref = ConanFileReference.loads("my_pkg/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder1")
        cpp_info.name = "MyPkG"
        self.conanfile.deps_cpp_info.add(ref.name, cpp_info)
        ref = ConanFileReference.loads("my_pkg2/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder2")
        cpp_info.name = "MyPkG2"
        cpp_info.public_deps = ["my_pkg"]
        self.conanfile.deps_cpp_info.add(ref.name, cpp_info)

    def test_cmake(self):
        generator = CMakeGenerator(self.conanfile)
        content = generator.content
        self.assertIn("set(CONAN_DEPENDENCIES my_pkg my_pkg2)", content)
        content = content.replace("set(CONAN_DEPENDENCIES my_pkg my_pkg2)", "")
        self.assertNotIn("my_pkg", content)
        self.assertNotIn("MY_PKG", content)
        self.assertIn('add_library(CONAN_PKG::MyPkG INTERFACE IMPORTED)', content)
        self.assertIn('add_library(CONAN_PKG::MyPkG2 INTERFACE IMPORTED)', content)
        self.assertNotIn('CONAN_PKG::my_pkg', content)
        self.assertNotIn('CONAN_PKG::my_pkg2', content)

    def test_cmake_multi(self):
        generator = CMakeMultiGenerator(self.conanfile)
        content = generator.content
        self.assertIn("set(CONAN_DEPENDENCIES_DEBUG my_pkg my_pkg2)",
                      content["conanbuildinfo_debug.cmake"])
        self.assertNotIn("my_pkg", content["conanbuildinfo_multi.cmake"])
        self.assertNotIn("MY_PKG", content["conanbuildinfo_multi.cmake"])
        self.assertIn('add_library(CONAN_PKG::MyPkG INTERFACE IMPORTED)',
                      content["conanbuildinfo_multi.cmake"])
        self.assertIn('add_library(CONAN_PKG::MyPkG2 INTERFACE IMPORTED)',
                      content["conanbuildinfo_multi.cmake"])
        self.assertNotIn('CONAN_PKG::my_pkg', content["conanbuildinfo_multi.cmake"])
        self.assertNotIn('CONAN_PKG::my_pkg2', content["conanbuildinfo_multi.cmake"])

    def test_cmake_find_package(self):
        generator = CMakeFindPackageGenerator(self.conanfile)
        content = generator.content
        self.assertIn("FindMyPkG.cmake", content.keys())
        self.assertIn("FindMyPkG2.cmake", content.keys())
        self.assertNotIn("my_pkg", content["FindMyPkG.cmake"])
        self.assertNotIn("MY_PKG", content["FindMyPkG.cmake"])
        self.assertNotIn("my_pkg", content["FindMyPkG2.cmake"])
        self.assertNotIn("MY_PKG", content["FindMyPkG2.cmake"])
        self.assertIn("add_library(MyPkG::MyPkG INTERFACE IMPORTED)", content["FindMyPkG.cmake"])
        self.assertIn("add_library(MyPkG2::MyPkG2 INTERFACE IMPORTED)", content["FindMyPkG2.cmake"])
        self.assertIn("find_dependency(MyPkG REQUIRED)", content["FindMyPkG2.cmake"])

    def test_cmake_find_package_multi(self):
        generator = CMakeFindPackageMultiGenerator(self.conanfile)
        content = generator.content
        six.assertCountEqual(self, ['MyPkG2Targets.cmake', 'MyPkGConfig.cmake', 'MyPkG2Config.cmake',
                                    'MyPkGTargets.cmake', 'MyPkGTarget-debug.cmake',
                                    'MyPkG2Target-debug.cmake', 'MyPkGConfigVersion.cmake',
                                    'MyPkG2ConfigVersion.cmake'], content.keys())
        self.assertNotIn("my_pkg", content["MyPkGConfig.cmake"])
        self.assertNotIn("MY_PKG", content["MyPkGConfig.cmake"])
        self.assertNotIn("my_pkg", content["MyPkG2Config.cmake"])
        self.assertNotIn("MY_PKG", content["MyPkG2Config.cmake"])
        self.assertIn("add_library(MyPkG::MyPkG INTERFACE IMPORTED)",
                      content["MyPkGTargets.cmake"])
        self.assertIn("add_library(MyPkG2::MyPkG2 INTERFACE IMPORTED)",
                      content["MyPkG2Targets.cmake"])
        self.assertIn("find_dependency(MyPkG REQUIRED NO_MODULE)", content["MyPkG2Config.cmake"])


class CMakeCppInfoNamesTest(unittest.TestCase):
    """
    Test cpp_info.names["generator"] values are applied in generators instead of the cpp_info.name
    """

    def setUp(self):
        self.conanfile = ConanFile(Mock(), None)
        settings = _MockSettings(build_type="Debug")
        self.conanfile.initialize(settings, EnvValues())
        ref = ConanFileReference.loads("my_pkg/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder1")
        cpp_info.name = "MyPkG"
        cpp_info.names["cmake"] = "MyCMakeName"
        cpp_info.names["cmake_multi"] = "MyCMakeMultiName"
        cpp_info.names["cmake_find_package"] = "MyCMakeFindPackageName"
        cpp_info.names["cmake_find_package_multi"] = "MyCMakeFindPackageMultiName"
        self.conanfile.deps_cpp_info.add(ref.name, cpp_info)
        ref = ConanFileReference.loads("my_pkg2/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder2")
        cpp_info.name = "MyPkG2"
        cpp_info.names["cmake"] = "MyCMakeName2"
        cpp_info.names["cmake_multi"] = "MyCMakeMultiName2"
        cpp_info.names["cmake_find_package"] = "MyCMakeFindPackageName2"
        cpp_info.names["cmake_find_package_multi"] = "MyCMakeFindPackageMultiName2"
        cpp_info.public_deps = ["my_pkg"]
        self.conanfile.deps_cpp_info.add(ref.name, cpp_info)

    def test_cmake(self):
        generator = CMakeGenerator(self.conanfile)
        content = generator.content
        self.assertNotIn("MyPkG", content)
        self.assertNotIn("MyPkG2", content)
        self.assertIn('add_library(CONAN_PKG::MyCMakeName INTERFACE IMPORTED)', content)
        self.assertIn('add_library(CONAN_PKG::MyCMakeName2 INTERFACE IMPORTED)', content)

    def test_cmake_multi(self):
        generator = CMakeMultiGenerator(self.conanfile)
        content = generator.content
        self.assertNotIn("MyPkG", content["conanbuildinfo_multi.cmake"])
        self.assertNotIn("MyPkG2", content["conanbuildinfo_multi.cmake"])
        self.assertIn('add_library(CONAN_PKG::MyCMakeMultiName INTERFACE IMPORTED)',
                      content["conanbuildinfo_multi.cmake"])
        self.assertIn('add_library(CONAN_PKG::MyCMakeMultiName2 INTERFACE IMPORTED)',
                      content["conanbuildinfo_multi.cmake"])

    def test_cmake_find_package(self):
        generator = CMakeFindPackageGenerator(self.conanfile)
        content = generator.content
        self.assertIn("FindMyCMakeFindPackageName.cmake", content.keys())
        self.assertIn("FindMyCMakeFindPackageName2.cmake", content.keys())
        self.assertNotIn("MyPkG", content["FindMyCMakeFindPackageName.cmake"])
        self.assertNotIn("MyPkG2", content["FindMyCMakeFindPackageName2.cmake"])
        self.assertIn("add_library(MyCMakeFindPackageName::MyCMakeFindPackageName INTERFACE IMPORTED)",
                      content["FindMyCMakeFindPackageName.cmake"])
        self.assertIn("add_library(MyCMakeFindPackageName2::MyCMakeFindPackageName2 INTERFACE IMPORTED)",
                      content["FindMyCMakeFindPackageName2.cmake"])
        self.assertIn("find_dependency(MyCMakeFindPackageName REQUIRED)",
                      content["FindMyCMakeFindPackageName2.cmake"])

    def test_cmake_find_package_multi(self):
        generator = CMakeFindPackageMultiGenerator(self.conanfile)
        content = generator.content
        six.assertCountEqual(self, ['MyCMakeFindPackageMultiName2Targets.cmake',
                                    'MyCMakeFindPackageMultiNameConfig.cmake',
                                    'MyCMakeFindPackageMultiName2Config.cmake',
                                    'MyCMakeFindPackageMultiNameTargets.cmake',
                                    'MyCMakeFindPackageMultiNameTarget-debug.cmake',
                                    'MyCMakeFindPackageMultiName2Target-debug.cmake',
                                    'MyCMakeFindPackageMultiNameConfigVersion.cmake',
                                    'MyCMakeFindPackageMultiName2ConfigVersion.cmake'],
                             content.keys())
        self.assertNotIn("MyPkG", content["MyCMakeFindPackageMultiNameConfig.cmake"])
        self.assertNotIn("MyPkG", content["MyCMakeFindPackageMultiName2Config.cmake"])
        self.assertIn(
            "add_library(MyCMakeFindPackageMultiName::MyCMakeFindPackageMultiName INTERFACE IMPORTED)",
            content["MyCMakeFindPackageMultiNameTargets.cmake"])
        self.assertIn(
            "add_library(MyCMakeFindPackageMultiName2::MyCMakeFindPackageMultiName2 INTERFACE IMPORTED)",
            content["MyCMakeFindPackageMultiName2Targets.cmake"])
        self.assertIn("find_dependency(MyCMakeFindPackageMultiName REQUIRED NO_MODULE)",
                      content["MyCMakeFindPackageMultiName2Config.cmake"])


class CMakeBuildModulesTest(unittest.TestCase):

    def setUp(self):
        settings_mock = _MockSettings(build_type="Release")
        self.conanfile = ConanFile(Mock(), None)
        self.conanfile.initialize(settings_mock, EnvValues())
        ref = ConanFileReference.loads("my_pkg/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder1")
        cpp_info.filter_empty = False  # For testing purposes only
        cpp_info.name = ref.name
        cpp_info.build_modules["cmake"] = ["my-module.cmake"]
        cpp_info.build_modules["cmake_multi"] = ["my-module.cmake"]
        cpp_info.build_modules["cmake_find_package"] = ["my-module.cmake"]
        cpp_info.build_modules["cmake_find_package_multi"] = ["my-module.cmake"]
        self.conanfile.deps_cpp_info.add(ref.name, cpp_info)
        ref = ConanFileReference.loads("my_pkg2/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder2")
        cpp_info.filter_empty = False  # For testing purposes only
        cpp_info.name = ref.name
        cpp_info.build_modules["cmake"] = ["other-mod.cmake", "not-a-cmake-module.pc"]
        cpp_info.build_modules["cmake_multi"] = ["other-mod.cmake", "not-a-cmake-module.pc"]
        cpp_info.build_modules["cmake_find_package"] = ["other-mod.cmake", "not-a-cmake-module.pc"]
        cpp_info.build_modules["cmake_find_package_multi"] = ["other-mod.cmake",
                                                              "not-a-cmake-module.pc"]
        cpp_info.release.build_modules["cmake"] = ["release-mod.cmake"]
        cpp_info.release.build_modules["cmake_multi"] = ["release-mod.cmake"]
        cpp_info.release.build_modules["cmake_find_package"] = ["release-mod.cmake"]
        cpp_info.release.build_modules["cmake_find_package_multi"] = ["release-mod.cmake"]
        self.conanfile.deps_cpp_info.add(ref.name, cpp_info)

    def test_cmake(self):
        generator = CMakeGenerator(self.conanfile)
        content = generator.content
        self.assertIn('set(CONAN_BUILD_MODULES_PATHS "dummy_root_folder1/my-module.cmake"'
                      '\n\t\t\t"dummy_root_folder2/other-mod.cmake"'
                      '\n\t\t\t"dummy_root_folder2/not-a-cmake-module.pc" '
                      '${CONAN_BUILD_MODULES_PATHS})',
                      content)
        self.assertIn('set(CONAN_BUILD_MODULES_PATHS_MY_PKG "dummy_root_folder1/my-module.cmake")',
                      content)
        self.assertIn('set(CONAN_BUILD_MODULES_PATHS_MY_PKG2 "dummy_root_folder2/other-mod.cmake"'
                      '\n\t\t\t"dummy_root_folder2/not-a-cmake-module.pc")',
                      content)
        self.assertIn("macro(conan_include_build_modules)", content)
        self.assertIn("conan_include_build_modules()", content)

    def test_cmake_multi(self):
        generator = CMakeMultiGenerator(self.conanfile)
        content = generator.content
        self.assertIn('set(CONAN_BUILD_MODULES_PATHS_RELEASE '
                      '"dummy_root_folder1/my-module.cmake"\n\t\t\t'
                      '"dummy_root_folder2/other-mod.cmake"\n\t\t\t'
                      '"dummy_root_folder2/not-a-cmake-module.pc"\n\t\t\t'
                      '"dummy_root_folder2/release-mod.cmake" '
                      '${CONAN_BUILD_MODULES_PATHS_RELEASE})',
                      content["conanbuildinfo_release.cmake"])
        self.assertIn('set(CONAN_BUILD_MODULES_PATHS_MY_PKG_RELEASE '
                      '"dummy_root_folder1/my-module.cmake")',
                      content["conanbuildinfo_release.cmake"])
        self.assertIn('set(CONAN_BUILD_MODULES_PATHS_MY_PKG2_RELEASE '
                      '"dummy_root_folder2/other-mod.cmake"\n\t\t\t'
                      '"dummy_root_folder2/not-a-cmake-module.pc"\n\t\t\t'
                      '"dummy_root_folder2/release-mod.cmake")',
                      content["conanbuildinfo_release.cmake"])
        self.assertIn("macro(conan_include_build_modules)", content["conanbuildinfo_multi.cmake"])
        self.assertIn("conan_include_build_modules()", content["conanbuildinfo_multi.cmake"])

    def test_cmake_find_package(self):
        generator = CMakeFindPackageGenerator(self.conanfile)
        content = generator.content
        self.assertIn("Findmy_pkg.cmake", content.keys())
        self.assertIn("Findmy_pkg2.cmake", content.keys())
        self.assertIn('set(CMAKE_MODULE_PATH "dummy_root_folder1/" ${CMAKE_MODULE_PATH})',
                      content["Findmy_pkg.cmake"])
        self.assertIn('set(CMAKE_PREFIX_PATH "dummy_root_folder1/" ${CMAKE_PREFIX_PATH})',
                      content["Findmy_pkg.cmake"])
        self.assertIn('set(CMAKE_MODULE_PATH "dummy_root_folder2/" ${CMAKE_MODULE_PATH})',
                      content["Findmy_pkg2.cmake"])
        self.assertIn('set(CMAKE_PREFIX_PATH "dummy_root_folder2/" ${CMAKE_PREFIX_PATH})',
                      content["Findmy_pkg2.cmake"])
        self.assertIn('set(my_pkg_BUILD_MODULES_PATHS "dummy_root_folder1/my-module.cmake")',
                      content["Findmy_pkg.cmake"])
        self.assertIn('set(my_pkg2_BUILD_MODULES_PATHS '
                      '"dummy_root_folder2/other-mod.cmake"\n\t\t\t'
                      '"dummy_root_folder2/not-a-cmake-module.pc"\n\t\t\t'
                      '"dummy_root_folder2/release-mod.cmake")',
                      content["Findmy_pkg2.cmake"])

    def test_cmake_find_package_multi(self):
        generator = CMakeFindPackageMultiGenerator(self.conanfile)
        content = generator.content
        self.assertIn('set(my_pkg_BUILD_MODULES_PATHS_RELEASE "dummy_root_folder1/my-module.cmake")',
                      content["my_pkgTarget-release.cmake"])
        self.assertIn('set(my_pkg2_BUILD_MODULES_PATHS_RELEASE "dummy_root_folder2/other-mod.cmake"'
                      '\n\t\t\t"dummy_root_folder2/not-a-cmake-module.pc"'
                      '\n\t\t\t"dummy_root_folder2/release-mod.cmake")',
                      content["my_pkg2Target-release.cmake"])



import os
import textwrap
import unittest

from conans.model.ref import ConanFileReference, PackageReference
from conans.test.utils.tools import TestClient

conanfile = """from conans import ConanFile

class TestConan(ConanFile):
    name = "%s"
    version = "0.1"
    exports = "*"
    def package(self):
        self.copy("*")
"""

conanfile_txt = """
[requires]
%s
"""


class ImportTest(unittest.TestCase):
    def _set_up(self):
        client = TestClient()
        client.save({"conanfile.py": conanfile % "LibA",
                     "LICENSE.txt": "LicenseA"})
        client.run("export . lasote/testing")

        client.save({"conanfile.py": conanfile % "LibB" + "    requires='LibA/0.1@lasote/testing'",
                     "LICENSE.md": "LicenseB"}, clean_first=True)
        client.run("export . lasote/testing")

        client.save({"conanfile.py": conanfile % "LibC" + "    requires='LibB/0.1@lasote/testing'",
                     "license.txt": "LicenseC"}, clean_first=True)
        client.run("export . lasote/testing")
        return client

    def test_repackage(self):
        client = self._set_up()
        conanfile = """from conans import ConanFile
class TestConan(ConanFile):
    requires='LibC/0.1@lasote/testing'
    keep_imports = True
    def imports(self):
        self.copy("license*", dst="licenses", folder=True, ignore_case=True)

    def package(self):
        self.copy("*")
"""
        client.save({"conanfile.py": conanfile}, clean_first=True)
        client.run("create . Pkg/0.1@user/testing --build=missing")
        self.assertIn("Pkg/0.1@user/testing package(): Packaged 1 '.md' file: LICENSE.md",
                      client.out)
        pref = PackageReference(ConanFileReference.loads("Pkg/0.1@user/testing"),
                                "e6f2dac07251ad9958120a7f7c324366fb3b6f2a")
        pkg_folder = client.cache.package_layout(pref.ref).package(pref)
        self.assertTrue(os.path.exists(os.path.join(pkg_folder, "licenses/LibA/LICENSE.txt")))
        self.assertTrue(os.path.exists(os.path.join(pkg_folder, "licenses/LibB/LICENSE.md")))
        self.assertTrue(os.path.exists(os.path.join(pkg_folder, "licenses/LibC/license.txt")))

    def test_imports_folders(self):
        client = self._set_up()

        testconanfile = conanfile % "LibD" + "    requires='LibC/0.1@lasote/testing'"
        testconanfile += """
    def imports(self):
        self.copy("license*", dst="licenses", folder=True, ignore_case=True)
        import os
        self.output.info("IMPORTED FOLDERS: %s " % sorted(os.listdir(self.imports_folder)))
"""
        client.save({"conanfile.py": testconanfile}, clean_first=True)
        client.run("install . --build=missing")
        self.assertIn("IMPORTED FOLDERS: [", client.out)
        self.assertEqual(client.load("licenses/LibA/LICENSE.txt"), "LicenseA")
        self.assertEqual(client.load("licenses/LibB/LICENSE.md"), "LicenseB")
        self.assertEqual(client.load("licenses/LibC/license.txt"), "LicenseC")

    def test_imports_folders_txt(self):
        client = self._set_up()

        conanfile = """[requires]
LibC/0.1@lasote/testing
[imports]
., license* -> licenses @ folder=True, ignore_case=True, excludes=*.md # comment
"""
        client.save({"conanfile.txt": conanfile}, clean_first=True)
        client.run("install . --build=missing")
        self.assertEqual(client.load("licenses/LibA/LICENSE.txt"), "LicenseA")
        self.assertFalse(os.path.exists(os.path.join(client.current_folder,
                                                     "licenses/LibB/LICENSE.md")))
        self.assertEqual(client.load("licenses/LibC/license.txt"), "LicenseC")

    def test_imports_wrong_args_txt(self):
        client = TestClient()
        conanfile = """
[imports]
., license* -> lic@myfolder @ something
"""
        client.save({"conanfile.txt": conanfile})
        client.run("install .", assert_error=True)
        self.assertIn("Wrong imports argument 'something'. Need a 'arg=value' pair.", client.out)

    def test_imports_wrong_line_txt(self):
        client = TestClient()
        conanfile = """
[imports]
., license*
"""
        client.save({"conanfile.txt": conanfile})
        client.run("install .", assert_error=True)
        self.assertIn("Wrong imports line: ., license*", client.out)
        self.assertIn("Use syntax: path, pattern -> local-folder", client.out)

    def test_imports_folders_extrachars_txt(self):
        # https://github.com/conan-io/conan/issues/4524
        client = self._set_up()

        conanfile = """[requires]
LibC/0.1@lasote/testing
[imports]
., license* -> lic@myfolder @ folder=True, ignore_case=True, excludes=*.md # comment
., *.md -> lic@myfolder @ # This is mandatory, otherwise it will fail
"""
        client.save({"conanfile.txt": conanfile}, clean_first=True)
        client.run("install . --build=missing")
        self.assertEqual(client.load("lic@myfolder/LibA/LICENSE.txt"), "LicenseA")
        self.assertEqual(client.load("lic@myfolder/LICENSE.md"), "LicenseB")
        self.assertEqual(client.load("lic@myfolder/LibC/license.txt"), "LicenseC")

    def test_conanfile_txt_multi_excludes(self):
        # https://github.com/conan-io/conan/issues/2293
        client = TestClient()
        pkg_conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                exports_sources = "*"
                def package(self):
                    self.copy("*.dll", dst="bin")
            """)
        client.save({"conanfile.py": pkg_conanfile,
                     "a.dll": "",
                     "Foo/b.dll": "",
                     "Baz/b.dll": ""})
        client.run("create . Pkg/0.1@user/testing")

        consumer = textwrap.dedent("""
            [requires]
            Pkg/0.1@user/testing
            [imports]
            bin, *.dll ->  @ excludes=Foo/*.dll Baz/*.dll
            """)
        client.save({"conanfile.txt": consumer}, clean_first=True)
        client.run("install . --build=missing")
        self.assertTrue(os.path.exists(os.path.join(client.current_folder, "a.dll")))
        self.assertFalse(os.path.exists(os.path.join(client.current_folder, "Foo/b.dll")))
        self.assertFalse(os.path.exists(os.path.join(client.current_folder, "Baz/b.dll")))

    def test_imports_keep_path(self):
        client = TestClient()
        client.save({"conanfile.py": conanfile % "LibC",
                     "path/to/license.txt": "LicenseC"}, clean_first=True)
        client.run("export . lasote/testing")

        # keep_path = True AND conanfile.py
        testconanfile = conanfile % "LibD" + "    requires='LibC/0.1@lasote/testing'"
        testconanfile += """
    def imports(self):
        self.copy("*license*", dst="licenses", keep_path=True)
"""
        client.save({"conanfile.py": testconanfile}, clean_first=True)
        client.run("install . --build=missing")
        self.assertTrue(os.path.exists(os.path.join(client.current_folder,
                                                    "licenses/path/to/license.txt")))

        # keep_path = False AND conanfile.py
        testconanfile = testconanfile.replace("keep_path=True", "keep_path=False")
        client.save({"conanfile.py": testconanfile}, clean_first=True)
        client.run("install . --build=missing")
        self.assertTrue(os.path.exists(os.path.join(client.current_folder,
                                                    "licenses/license.txt")))

        # keep_path = True AND conanfile.txt
        testconanfile = conanfile_txt % "LibC/0.1@lasote/testing"
        testconanfile += """
[imports]:
., *license* -> ./licenses @ keep_path=True
"""
        client.save({"conanfile.txt": testconanfile}, clean_first=True)
        client.run("install . --build=missing")
        self.assertTrue(os.path.exists(os.path.join(client.current_folder,
                                                    "licenses/path/to/license.txt")))

        # keep_path = False AND conanfile.txt
        testconanfile = testconanfile.replace("keep_path=True", "keep_path=False")
        client.save({"conanfile.txt": testconanfile}, clean_first=True)
        client.run("install . --build=missing")
        self.assertTrue(os.path.exists(os.path.join(client.current_folder, "licenses")))
        self.assertTrue(os.path.exists(os.path.join(client.current_folder, "licenses/license.txt")))

    def test_wrong_path_sep(self):
        # https://github.com/conan-io/conan/issues/7856
        pkg_conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conans.tools import save
            import os

            class TestConan(ConanFile):
                def package(self):
                    save(os.path.join(self.package_folder, "licenses/LICENSE"), "mylicense in file")
                    save(os.path.join(self.package_folder, "licenses/README"), "myreadme in file")
            """)
        client = TestClient()
        client.save({"conanfile.py": pkg_conanfile})
        client.run("create . pkg/0.1@")
        consumer_conanfile = textwrap.dedent(r"""
            from conans import ConanFile
            import platform

            class TestConan(ConanFile):
                requires = "pkg/0.1"

                def imports(self):
                    if platform.system() == "Windows":
                        self.copy("licenses\\LICENSE", dst="deps_licenses", root_package="pkg")
                    else:
                        self.copy("licenses/LICENSE", dst="deps_licenses", root_package="pkg")
                    self.copy("licenses/README", dst="deps_licenses", root_package="pkg")
            """)
        client.save({"conanfile.py": consumer_conanfile})
        client.run("install .")
        pkg_license = client.load("deps_licenses/licenses/LICENSE")
        self.assertEqual(pkg_license, "mylicense in file")
        readme_license = client.load("deps_licenses/licenses/README")
        self.assertEqual(readme_license, "myreadme in file")



import unittest

import mock
from parameterized import parameterized

from conans.client.conf.detect import _gcc_compiler
from conans.test.utils.mocks import TestBufferConanOutput


class GCCCompilerTestCase(unittest.TestCase):

    @parameterized.expand([("10",), ("4.2",), ('7', )])
    def test_detect_gcc_10(self, version):
        output = TestBufferConanOutput()
        with mock.patch("platform.system", return_value="Linux"):
            with mock.patch("conans.client.conf.detect.detect_runner", return_value=(0, version)):
                compiler, installed_version = _gcc_compiler(output)
        self.assertEqual(compiler, 'gcc')
        self.assertEqual(installed_version, version)



import platform
import re
import textwrap

import pytest

from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() not in ["Linux", "Darwin"], reason="Autotools")
def test_link_lib_correct_order():
    client = TestClient()
    liba = GenConanfile().with_name("liba").with_version("0.1")
    libb = GenConanfile().with_name("libb").with_version("0.1").with_require("liba/0.1")
    libc = GenConanfile().with_name("libc").with_version("0.1").with_require("libb/0.1")
    consumer = GenConanfile().with_require("libc/0.1")
    client.save({"liba.py": liba, "libb.py": libb, "libc.py": libc, "consumer.py": consumer})
    client.run("create liba.py")
    client.run("create libb.py")
    client.run("create libc.py")
    client.run("install consumer.py -g AutotoolsDeps")
    deps = client.load("conanautotoolsdeps.sh")
    # check the libs are added in the correct order with this regex
    assert re.search("export LDFLAGS.*libc.*libb.*liba", deps)


@pytest.mark.skipif(platform.system() not in ["Linux", "Darwin"], reason="Autotools")
def test_cpp_info_aggregation():

    profile = textwrap.dedent("""
         [settings]
         build_type=Release
         arch=x86
         os=Macos
         compiler=gcc
         compiler.libcxx=libstdc++11
         compiler.version=7.1
         cppstd=17
    """)

    dep_conanfile = textwrap.dedent("""

    from conan import ConanFile

    class Dep(ConanFile):

        settings = "os", "arch", "compiler", "build_type"

        def package_info(self):
            self.cpp_info.includedirs = []
            self.cpp_info.includedirs.append("path/includes/{}".format(self.name))
            self.cpp_info.includedirs.append("other\\include\\path\\{}".format(self.name))
            # To test some path in win, to be used with MinGW make or MSYS etc
            self.cpp_info.libdirs = []
            self.cpp_info.libdirs.append("one\\lib\\path\\{}".format(self.name))
            self.cpp_info.libs = []
            self.cpp_info.libs.append("{}_onelib".format(self.name))
            self.cpp_info.libs.append("{}_twolib".format(self.name))
            self.cpp_info.defines = []
            self.cpp_info.defines.append("{}_onedefinition".format(self.name))
            self.cpp_info.defines.append("{}_twodefinition".format(self.name))
            self.cpp_info.cflags = ["{}_a_c_flag".format(self.name)]
            self.cpp_info.cxxflags = ["{}_a_cxx_flag".format(self.name)]
            self.cpp_info.sharedlinkflags = ["{}_shared_link_flag".format(self.name)]
            self.cpp_info.exelinkflags = ["{}_exe_link_flag".format(self.name)]
            self.cpp_info.sysroot = "/path/to/folder/{}".format(self.name)
            self.cpp_info.frameworks = []
            self.cpp_info.frameworks.append("{}_oneframework".format(self.name))
            self.cpp_info.frameworks.append("{}_twoframework".format(self.name))
            self.cpp_info.system_libs = []
            self.cpp_info.system_libs.append("{}_onesystemlib".format(self.name))
            self.cpp_info.system_libs.append("{}_twosystemlib".format(self.name))
            self.cpp_info.frameworkdirs = []
            self.cpp_info.frameworkdirs.append("one/framework/path/{}".format(self.name))

    """)

    t = TestClient()
    t.save({"conanfile.py": dep_conanfile, "macos": profile})
    t.run("create . dep1/1.0@ --profile:host=macos")
    t.run("create . dep2/1.0@ --profile:host=macos")

    consumer = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.gnu import AutotoolsDeps

        class HelloConan(ConanFile):
            generators = "AutotoolsDeps"
            requires = "dep1/1.0", "dep2/1.0"
            settings = "os", "arch", "build_type", "compiler"

            def generate(self):
                deps = AutotoolsDeps(self)
                env = deps.environment

                # Customize the environment
                env.remove("LDFLAGS", "dep2_shared_link_flag")
                env.append("LDFLAGS", "OtherSuperStuff")

                env = deps.vars()
                # The contents are of course modified
                self.output.warn(env["CXXFLAGS"])

                # The topological order puts dep2 before dep1
                assert env["CXXFLAGS"] == 'dep2_a_cxx_flag dep1_a_cxx_flag'
                assert env["CFLAGS"] == 'dep2_a_c_flag dep1_a_c_flag'
                assert env["LIBS"] == "-ldep2_onelib -ldep2_twolib -ldep1_onelib -ldep1_twolib "\
                                      "-ldep2_onesystemlib -ldep2_twosystemlib "\
                                      "-ldep1_onesystemlib -ldep1_twosystemlib"

                assert env["LDFLAGS"].startswith('dep1_shared_link_flag dep2_exe_link_flag dep1_exe_link_flag -framework dep2_oneframework -framework dep2_twoframework ' \
                                     '-framework dep1_oneframework -framework dep1_twoframework ')
                assert 'OtherSuperStuff' in env["LDFLAGS"]
    """)

    t.save({"conanfile.py": consumer})
    t.run("create . consumer/1.0@ --profile:host=macos")



import os

from conans.client.cache.editable import EditablePackages
from conans.errors import ConanException
from conans.model.ref import ConanFileReference
from sense2.paths.package_layouts.package_cache_layout import PackageCacheLayout
from conans.paths.package_layouts.package_editable_layout import PackageEditableLayout
from conans.util.files import list_folder_subdirs, load, normalize, save, remove
from conans.util.locks import Lock
from conans.client.cache.cache import (ClientCache as ClientCache_co, _is_case_insensitive_os, )

CONAN_CONF = 'conan.conf'
CONAN_SETTINGS = "settings.yml"
LOCALDB = ".conan.db"
REMOTES = "remotes.json"
PROFILES_FOLDER = "profiles"
HOOKS_FOLDER = "hooks"
TEMPLATES_FOLDER = "templates"
GENERATORS_FOLDER = "generators"



if _is_case_insensitive_os():
    def _check_ref_case(ref, store_folder):
        if not os.path.exists(store_folder):
            return

        tmp = store_folder
        for part in ref.dir_repr().split("/"):
            items = os.listdir(tmp)
            try:
                idx = [item.lower() for item in items].index(part.lower())
                if part != items[idx]:
                    raise ConanException("Requested '{requested}', but found case incompatible"
                                         " recipe with name '{existing}' in the cache. Case"
                                         " insensitive filesystem can't manage this.\n Remove"
                                         " existing recipe '{existing}' and try again.".format(
                        requested=str(ref), existing=items[idx]
                    ))
                tmp = os.path.normpath(tmp + os.sep + part)
            except ValueError:
                return
else:
    def _check_ref_case(ref, store_folder):  # @UnusedVariable
        pass


class ClientCache(ClientCache_co):
    """ Class to represent/store/compute all the paths involved in the execution
    of conans commands. Accesses to real disk and reads/write things. (OLD client ConanPaths)
    """

    def package_layout(self, ref, short_paths=None):
        assert isinstance(ref, ConanFileReference), "It is a {}".format(type(ref))
        edited_ref = self.editable_packages.get(ref.copy_clear_rev())
        if edited_ref:
            conanfile_path = edited_ref["path"]
            layout_file = edited_ref["layout"]
            return PackageEditableLayout(os.path.dirname(conanfile_path), layout_file, ref,
                                         conanfile_path, edited_ref.get("output_folder"))
        else:
            _check_ref_case(ref, self.store)
            base_folder = os.path.normpath(os.path.join(self.store, ref.dir_repr()))
            return PackageCacheLayout(base_folder=base_folder, ref=ref,
                                      short_paths=short_paths, no_lock=self._no_locks())

    


import json
import textwrap
import time
import unittest

import pytest

from conans.model.graph_lock import LOCKFILE
from conans.test.utils.tools import TestClient, GenConanfile
from conans.util.env_reader import get_env


class GraphLockErrorsTest(unittest.TestCase):
    def test_missing_lock_error(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("PkgA", "0.1")})
        client.run("install . --lockfile=conan.lock", assert_error=True)
        self.assertIn("ERROR: Missing lockfile in", client.out)

    def test_update_different_profile(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("PkgA", "0.1")})
        client.run("install . -if=conf1 -s os=Windows")
        client.run("install . -if=conf2 -s os=Linux")
        client.run("lock update conf1/conan.lock conf2/conan.lock", assert_error=True)
        self.assertIn("Profiles of lockfiles are different", client.out)
        self.assertIn("os=Windows", client.out)
        self.assertIn("os=Linux", client.out)

    def test_try_to_pass_profile(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("PkgA", "0.1")})
        client.run("lock create conanfile.py --lockfile-out=conan.lock")
        client.run("install . --lockfile=conan.lock -s os=Windows", assert_error=True)
        self.assertIn("ERROR: Cannot use profile, settings, options, env or conf 'host' "
                      "when using lockfile", client.out)
        client.run("install . --lockfile=conan.lock -pr=default", assert_error=True)
        self.assertIn("ERROR: Cannot use profile, settings, options, env or conf 'host' "
                      "when using lockfile", client.out)
        client.run("install . --lockfile=conan.lock -o myoption=default", assert_error=True)
        self.assertIn("ERROR: Cannot use profile, settings, options, env or conf 'host' "
                      "when using lockfile", client.out)
        client.run("install . --lockfile=conan.lock -s:b os=Windows", assert_error=True)
        self.assertIn("ERROR: Cannot use profile, settings, options, env or conf 'build' "
                      "when using lockfile", client.out)
        client.run("install . --lockfile=conan.lock --profile:build=default", assert_error=True)
        self.assertIn("ERROR: Cannot use profile, settings, options, env or conf 'build' "
                      "when using lockfile", client.out)
        client.run("install . --lockfile=conan.lock -o:b myoption=default", assert_error=True)
        self.assertIn("ERROR: Cannot use profile, settings, options, env or conf 'build' "
                      "when using lockfile", client.out)
        client.run("install . --lockfile=conan.lock -c:b core:required_conan_version=>=1.36",
                   assert_error=True)
        self.assertIn("ERROR: Cannot use profile, settings, options, env or conf 'build' "
                      "when using lockfile", client.out)

    def test_error_old_format(self):
        client = TestClient()
        client.save({"conanfile.txt": ""})
        client.run("install .")
        lockfile = client.load("conan.lock")
        lockfile = lockfile.replace('"0.4"', '"0.1"').replace('"0"', '"UUID"')
        client.save({"conan.lock": lockfile})
        client.run("install . --lockfile=conan.lock", assert_error=True)
        self.assertIn("This lockfile was created with an incompatible version", client.out)

    def test_error_no_find(self):
        client = TestClient()
        client.save({"consumer.txt": ""})
        client.run("lock create consumer.txt --lockfile-out=output.lock")
        client.run("install consumer.txt --lockfile=output.lock")

        client.save({"consumer.py": GenConanfile()})
        client.run("lock create consumer.py --lockfile-out=output.lock "
                   "--name=name --version=version")
        client.run("install consumer.py name/version@ --lockfile=output.lock")
        self.assertIn("consumer.py (name/version): Generated graphinfo", client.out)

    @staticmethod
    def test_error_no_filename():
        # https://github.com/conan-io/conan/issues/8675
        client = TestClient()
        client.save({"consumer.txt": ""})
        client.run("lock create .", assert_error=True)
        assert "RROR: Path argument must include filename like 'conanfile.py'" in client.out

    def test_commands_cannot_create_lockfile(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("PkgA", "0.1")})
        client.run("export . --lockfile-out=conan.lock", assert_error=True)
        self.assertIn("ERROR: lockfile_out cannot be specified if lockfile is not defined",
                      client.out)
        client.run("install . --lockfile-out=conan.lock", assert_error=True)
        self.assertIn("ERROR: lockfile_out cannot be specified if lockfile is not defined",
                      client.out)
        client.run("info . --lockfile-out=conan.lock", assert_error=True)
        self.assertIn("ERROR: lockfile_out cannot be specified if lockfile is not defined",
                      client.out)
        client.run("create . --lockfile-out=conan.lock", assert_error=True)
        self.assertIn("ERROR: lockfile_out cannot be specified if lockfile is not defined",
                      client.out)

    def test_cannot_create_twice(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("PkgA", "0.1")})
        client.run("lock create conanfile.py --lockfile-out=conan.lock")
        client.run("create . --lockfile=conan.lock --lockfile-out=conan.lock")
        client.run("install PkgA/0.1@ --build=PkgA --lockfile=conan.lock --lockfile-out=conan.lock",
                   assert_error=True)
        rev = "#fa090239f8ba41ad559f8e934494ee2a" if client.cache.config.revisions_enabled else ""
        self.assertIn("Cannot build 'PkgA/0.1{}' because it is already locked".format(rev),
                      client.out)

        client.run("create . --lockfile=conan.lock --lockfile-out=conan.lock", assert_error=True)
        self.assertIn("ERROR: Attempt to modify locked PkgA/0.1", client.out)


class GraphLockConanfileTXTTest(unittest.TestCase):
    def test_conanfile_txt(self):
        client = TestClient()
        client.save({"conanfile.txt": ""})
        client.run("install .")
        client.run("install . --lockfile=conan.lock")
        self.assertIn("Using lockfile", client.out)

    def test_conanfile_txt_deps(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . pkg/0.1@user/testing")

        client2 = TestClient(cache_folder=client.cache_folder)
        client2.save({"conanfile.txt": "[requires]\npkg/[>0.0]@user/testing"})
        client2.run("install .")
        self.assertIn("pkg/0.1@user/testing from local cache - Cache", client2.out)

        client.run("create . pkg/0.2@user/testing")

        client2.run("install . --lockfile=conan.lock")
        self.assertIn("pkg/0.1@user/testing from local cache - Cache", client2.out)
        self.assertNotIn("pkg/0.2", client2.out)
        client2.run("install .")
        self.assertIn("pkg/0.2@user/testing from local cache - Cache", client2.out)
        self.assertNotIn("pkg/0.1", client2.out)


class ReproducibleLockfiles(unittest.TestCase):
    def test_reproducible_lockfile(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("PkgA", "0.1")})
        client.run("create . PkgA/0.1@user/channel")

        # Use a consumer with a version range
        client.save({"conanfile.py":
                     GenConanfile("PkgB", "0.1").with_require("PkgA/[>=0.1]@user/channel")})
        client.run("lock create conanfile.py --lockfile-out=lock1.lock")
        lockfile = client.load("lock1.lock")
        client.run("lock create conanfile.py --lockfile-out=lock2.lock")
        lockfile2 = client.load("lock2.lock")
        self.assertEqual(lockfile, lockfile2)
        # the default lockfile-out is conan.lock
        client.run("lock create conanfile.py")
        conanlock = client.load("conan.lock")
        self.assertEqual(lockfile, conanlock)
        # different commands still generate identical lock
        client.run("install .")
        info_lock = client.load("conan.lock")
        self.assertEqual(lockfile, info_lock)

    def test_reproducible_lockfile_txt(self):
        client = TestClient()
        client.save({"conanfile.txt": ""})
        client.run("install .")
        lockfile = client.load("conan.lock")
        client.run("install .")
        lockfile2 = client.load("conan.lock")
        self.assertEqual(lockfile, lockfile2)
        # check that the path to local conanfile.txt is relative, reproducible in other machine
        self.assertIn('"path": "conanfile.txt"', lockfile)


@pytest.mark.skipif(not get_env("TESTING_REVISIONS_ENABLED", False), reason="Only revisions")
class GraphLockRevisionTest(unittest.TestCase):
    rrev_b = "9b64caa2465f7660e6f613b7e87f0cd7"
    pkg_b_id = "5bf1ba84b5ec8663764a406f08a7f9ae5d3d5fb5"
    prev_b = "2ec4fb334e1b4f3fd0a6f66605066ac7"

    def setUp(self):
        client = TestClient(default_server_user=True)
        # Important to activate revisions
        self.client = client
        client.save({"conanfile.py": GenConanfile("PkgA", "0.1")})
        client.run("create . PkgA/0.1@user/channel")
        client.run("upload PkgA/0.1@user/channel --all")

        consumer = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                name = "PkgB"
                version = "0.1"
                requires = "PkgA/0.1@user/channel"
                def build(self):
                    self.output.info("BUILD DEP LIBS: %s!!" % ",".join(self.deps_cpp_info.libs))
                def package_info(self):
                    self.output.info("PACKAGE_INFO DEP LIBS: %s!!"
                                     % ",".join(self.deps_cpp_info.libs))
            """)
        client.save({"conanfile.py": str(consumer)})
        client.run("install . PkgB/0.1@user/channel")

        self._check_lock("PkgB/0.1@user/channel")

        # If we create a new PkgA revision, for example adding info
        pkga = GenConanfile("PkgA", "0.1")
        pkga.with_package_info(cpp_info={"libs": ["mylibPkgA0.1lib"]},
                               env_info={"MYENV": ["myenvPkgA0.1env"]})
        client.save({"conanfile.py": pkga})

        client.run("create . PkgA/0.1@user/channel")
        client.save({"conanfile.py": str(consumer)})

    def _check_lock(self, ref_b, pkg_b_id=None, prev_b=None):
        lockfile = self.client.load(LOCKFILE)
        lock_file_json = json.loads(lockfile)
        self.assertEqual(2, len(lock_file_json["graph_lock"]["nodes"]))
        pkga = lock_file_json["graph_lock"]["nodes"]["1"]
        self.assertEqual(pkga["ref"], "PkgA/0.1@user/channel#fa090239f8ba41ad559f8e934494ee2a")
        pkgb = lock_file_json["graph_lock"]["nodes"]["0"]
        self.assertEqual(pkgb["ref"], ref_b)
        self.assertEqual(pkgb.get("package_id"), pkg_b_id)
        self.assertEqual(pkgb.get("prev"), prev_b)

    def test_install_info_lock(self):
        # Normal install will use it (use install-folder to not change graph-info)
        client = self.client
        client.run("install . -if=tmp")  # Output graph_info to temporary
        client.run("build . -if=tmp")
        self.assertIn("conanfile.py (PkgB/0.1): BUILD DEP LIBS: mylibPkgA0.1lib!!", client.out)

        # Locked install will use PkgA/0.1
        # This is a bit weird, that is necessary to force the --update the get the rigth revision
        client.run("install . -g=cmake --lockfile=conan.lock --lockfile-out=conan.lock --update")
        self._check_lock("PkgB/0.1@user/channel")
        client.run("build .")
        self.assertIn("conanfile.py (PkgB/0.1@user/channel): BUILD DEP LIBS: !!", client.out)

        # Info also works
        client.run("info . --lockfile=conan.lock")
        self.assertIn("Revision: fa090239f8ba41ad559f8e934494ee2a", client.out)

    def test_export_lock(self):
        # locking a version range at export
        self.client.run("export . user/channel --lockfile=conan.lock --lockfile-out=conan.lock")
        self._check_lock("PkgB/0.1@user/channel#%s" % self.rrev_b)

    def test_create_lock(self):
        # Create is also possible
        client = self.client
        client.run("create . PkgB/0.1@user/channel --update --lockfile=conan.lock "
                   "--lockfile-out=conan.lock")
        self._check_lock("PkgB/0.1@user/channel#%s" % self.rrev_b, self.pkg_b_id, self.prev_b)

    def test_export_pkg(self):
        client = self.client
        # Necessary to clean previous revision
        client.run("remove * -f")
        client.run("export-pkg . PkgB/0.1@user/channel --lockfile=conan.lock "
                   "--lockfile-out=conan.lock")
        self._check_lock("PkgB/0.1@user/channel#%s" % self.rrev_b, self.pkg_b_id, self.prev_b)


@pytest.mark.skipif(not get_env("TESTING_REVISIONS_ENABLED", False), reason="Only revisions")
class RevisionsUpdateTest(unittest.TestCase):

    def test_revisions_update(self):
        # https://github.com/conan-io/conan/issues/7333
        client = TestClient(default_server_user=True)
        client.save({"conanfile.py": GenConanfile("PkgA", "0.1")})
        client.run("create . ")
        self.assertIn("PkgA/0.1: Exported revision: fa090239f8ba41ad559f8e934494ee2", client.out)
        client.run("upload * --all --confirm")
        client.run("search PkgA/0.1@ --revisions -r=default")
        self.assertIn("fa090239f8ba41ad559f8e934494ee2a", client.out)

        client2 = TestClient(servers=client.servers, users=client.users)
        client2.save({"conanfile.py": GenConanfile("PkgA", "0.1").with_build_msg("Building")})
        time.sleep(1)
        client2.run("create . ")
        self.assertIn("PkgA/0.1: Exported revision: 7f1110e1ae8d852b6d55f7f121864de6", client2.out)
        client2.run("upload * --all --confirm")
        client2.run("search PkgA/0.1@ --revisions -r=default")
        self.assertIn("fa090239f8ba41ad559f8e934494ee2a", client2.out)
        self.assertIn("7f1110e1ae8d852b6d55f7f121864de6", client2.out)

        client.save({"conanfile.py": GenConanfile("PkgB", "0.1").with_require("PkgA/0.1")})
        client.run("lock create conanfile.py --update --lockfile-out=conan.lock")
        self.assertIn("PkgA/0.1: Downloaded recipe revision 7f1110e1ae8d852b6d55f7f121864de6",
                      client.out)
        lockfile = client.load("conan.lock")
        self.assertIn("7f1110e1ae8d852b6d55f7f121864de6", lockfile)

        # Put again the old revision locally
        client.save({"conanfile.py": GenConanfile("PkgA", "0.1")})
        client.run("create . ")
        self.assertIn("PkgA/0.1: Exported revision: fa090239f8ba41ad559f8e934494ee2", client.out)

        # Local revisions are not updated, even with a lockfile, unless --update
        client.save({"conanfile.py": GenConanfile("PkgB", "0.1").with_require("PkgA/0.1")})
        client.run("install . --lockfile=conan.lock", assert_error=True)
        self.assertIn("ERROR: The 'fa090239f8ba41ad559f8e934494ee2a' revision recipe in the "
                      "local cache", client.out)
        client.run("install . --lockfile=conan.lock --update")
        self.assertIn("PkgA/0.1: Downloaded recipe revision 7f1110e1ae8d852b6d55f7f121864de6",
                      client.out)

    def test_version_ranges_revisions_update(self):
        # https://github.com/conan-io/conan/issues/7333
        client = TestClient(default_server_user=True)
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . PkgA/0.1@")
        self.assertIn("PkgA/0.1: Exported revision: f3367e0e7d170aa12abccb175fee5f97", client.out)
        client.run("upload * --all --confirm")
        client.run("search PkgA/0.1@ --revisions -r=default")
        self.assertIn("f3367e0e7d170aa12abccb175fee5f97", client.out)

        client2 = TestClient(servers=client.servers, users=client.users)
        client2.save({"conanfile.py": GenConanfile("PkgA")})
        client2.run("create . PkgA/0.2@")
        client2.run("upload * --all --confirm")
        client2.save({"conanfile.py": GenConanfile("PkgA", "0.2").with_build_msg("Building")})
        time.sleep(1)
        client2.run("create . ")
        self.assertIn("PkgA/0.2: Exported revision: 5e8148093372278be4e8d8e831d8bdb6", client2.out)
        client2.run("upload * --all --confirm")
        client2.run("search PkgA/0.2@ --revisions -r=default")
        self.assertIn("5e8148093372278be4e8d8e831d8bdb6", client2.out)
        self.assertIn("8ec297bab84c88218d1db36ffea97d0e", client2.out)

        client.save({"conanfile.py": GenConanfile("PkgB", "0.1").with_require("PkgA/[>=0.1]")})
        client.run("lock create conanfile.py --update --lockfile-out=conan.lock")
        self.assertIn("PkgA/0.2: Downloaded recipe revision 5e8148093372278be4e8d8e831d8bdb6",
                      client.out)
        lockfile = client.load("conan.lock")
        self.assertIn("5e8148093372278be4e8d8e831d8bdb6", lockfile)

        # Put again the old revision locally
        client.run("remove * -f")
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . PkgA/0.1@")
        self.assertIn("PkgA/0.1: Exported revision: f3367e0e7d170aa12abccb175fee5f97", client.out)

        client.save({"conanfile.py": GenConanfile("PkgB", "0.1").with_require("PkgA/[>=0.1]")})
        client.run("install . --lockfile=conan.lock")
        self.assertIn("PkgA/0.2: Downloaded recipe revision 5e8148093372278be4e8d8e831d8bdb6",
                      client.out)


class LockFileOptionsTest(unittest.TestCase):
    def test_options(self):
        client = TestClient()
        ffmpeg = textwrap.dedent("""
            from conans import ConanFile
            class FfmpegConan(ConanFile):
                options = {"variation": ["standard", "nano"]}
                default_options = {"variation": "standard"}

                def requirements(self):
                    variation = str(self.options.variation)
                    self.output.info("Requirements: Variation %s!!" % variation)
                    if self.options.variation == "standard":
                        self.requires("missingdep/1.0")
            """)

        variant = textwrap.dedent("""
            from conans import ConanFile
            class Meta(ConanFile):
                requires = "ffmpeg/1.0"
                default_options = {"ffmpeg:variation": "nano"}
            """)

        client.save({"ffmepg/conanfile.py": ffmpeg,
                     "variant/conanfile.py": variant})
        client.run("export ffmepg ffmpeg/1.0@")
        client.run("export variant nano/1.0@")

        client.run("lock create --reference=nano/1.0@ --build --lockfile-out=conan.lock")
        lockfile = client.load("conan.lock")
        self.assertIn('"options": "variation=nano"', lockfile)
        client.run("create ffmepg ffmpeg/1.0@ --build --lockfile=conan.lock")
        self.assertIn("ffmpeg/1.0: Requirements: Variation nano!!", client.out)

    def test_base_options(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile().with_option("shared", [True, False])
                                                   .with_default_option("shared", False)})
        client.run("create . pkg/0.1@")
        client.run("lock create --reference=pkg/0.1 --base --lockfile-out=pkg_base.lock")
        client.run("lock create --reference=pkg/0.1 --lockfile=pkg_base.lock "
                   "--lockfile-out=pkg.lock -o pkg:shared=True")
        pkg_lock = client.load("pkg.lock")
        self.assertIn('"options": "shared=True"', pkg_lock)
        client.run("lock create --reference=pkg/0.1 --lockfile=pkg_base.lock "
                   "--lockfile-out=pkg.lock -o pkg:shared=False")
        pkg_lock = client.load("pkg.lock")
        self.assertIn('"options": "shared=False"', pkg_lock)

    def test_config_option(self):
        # https://github.com/conan-io/conan/issues/7991
        client = TestClient()
        pahomqttc = textwrap.dedent("""
            from conans import ConanFile
            class PahoMQTCC(ConanFile):
                options = {"shared": [True, False]}
                default_options = {"shared": False}

                def config_options(self):
                    # This is weaker than "configure()", will be overwritten by downstream
                    self.options.shared = True
            """)

        pahomqttcpp = textwrap.dedent("""
            from conans import ConanFile
            class Meta(ConanFile):
                requires = "pahomqttc/1.0"
                def configure(self):
                    self.options["pahomqttc"].shared = False
            """)

        client.save({"pahomqttc/conanfile.py": pahomqttc,
                     "pahomqttcpp/conanfile.py": pahomqttcpp,
                     "consumer/conanfile.txt": "[requires]\npahomqttcpp/1.0"})
        client.run("export pahomqttc pahomqttc/1.0@")
        client.run("export pahomqttcpp pahomqttcpp/1.0@")

        client.run("install consumer/conanfile.txt --build -o paho-mqtt-c:shared=False")
        lockfile = client.load("conan.lock")
        self.assertIn('"options": "pahomqttc:shared=False"', lockfile)
        self.assertNotIn('shared=True', lockfile)
        client.run("install consumer/conanfile.txt --lockfile=conan.lock")

    def test_configure(self):
        # https://github.com/conan-io/conan/issues/7991
        client = TestClient()
        pahomqttc = textwrap.dedent("""
            from conans import ConanFile
            class PahoMQTCC(ConanFile):
                options = {"shared": [True, False]}
                default_options = {"shared": False}

                def configure(self):
                    self.options.shared = True
            """)

        pahomqttcpp = textwrap.dedent("""
            from conans import ConanFile
            class Meta(ConanFile):
                requires = "pahomqttc/1.0"
                def configure(self):
                    self.options["pahomqttc"].shared = False
            """)

        client.save({"pahomqttc/conanfile.py": pahomqttc,
                     "pahomqttcpp/conanfile.py": pahomqttcpp,
                     "consumer/conanfile.txt": "[requires]\npahomqttcpp/1.0"})
        client.run("export pahomqttc pahomqttc/1.0@")
        client.run("export pahomqttcpp pahomqttcpp/1.0@")

        client.run("install consumer/conanfile.txt --build -o paho-mqtt-c:shared=False")
        lockfile = client.load("conan.lock")
        self.assertIn('"options": "pahomqttc:shared=True"', lockfile)
        # Check the trailing ", to not get the profile one
        self.assertNotIn('shared=False"', lockfile)
        client.run("install consumer/conanfile.txt --lockfile=conan.lock")


class GraphInstallArgumentsUpdated(unittest.TestCase):

    def test_lockfile_argument_updated_install(self):
        # https://github.com/conan-io/conan/issues/6845
        # --lockfile parameter is not updated after install and
        # outputs results to conan.lock
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . somelib/1.0@")
        client.run("lock create --reference=somelib/1.0@ --lockfile-out=somelib.lock")
        previous_lock = client.load("somelib.lock")
        # This should fail, because somelib is locked
        client.run("install somelib/1.0@ --lockfile=somelib.lock --build somelib", assert_error=True)
        rev = "#f3367e0e7d170aa12abccb175fee5f97" if client.cache.config.revisions_enabled else ""
        self.assertIn("Cannot build 'somelib/1.0{}' because it "
                      "is already locked in the input lockfile".format(rev), client.out)
        new_lock = client.load("somelib.lock")
        self.assertEqual(previous_lock, new_lock)


class BuildLockedTest(unittest.TestCase):
    def test_build_locked_node(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . flac/1.0@")
        client.run("lock create --reference=flac/1.0@ --lockfile-out=conan.lock --build")
        lock = json.loads(client.load("conan.lock"))
        flac = lock["graph_lock"]["nodes"]["1"]
        if client.cache.config.revisions_enabled:
            ref = "flac/1.0#f3367e0e7d170aa12abccb175fee5f97"
            prev = "83c38d3b4e5f1b8450434436eec31b00"
        else:
            ref = "flac/1.0"
            prev = "0"
        self.assertEqual(flac["ref"], ref)
        self.assertEqual(flac["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
        self.assertIsNone(flac.get("prev"))

        client.run("install flac/1.0@ --lockfile=conan.lock --lockfile-out=output.lock")
        lock = json.loads(client.load("output.lock"))
        flac = lock["graph_lock"]["nodes"]["1"]
        self.assertEqual(flac["ref"], ref)
        self.assertEqual(flac["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
        self.assertIsNone(flac.get("prev"))

        client.run("install flac/1.0@ --build=flac --lockfile=conan.lock --lockfile-out=output.lock")
        lock = json.loads(client.load("output.lock"))
        flac = lock["graph_lock"]["nodes"]["1"]
        self.assertEqual(flac["ref"], ref)
        self.assertEqual(flac["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
        self.assertEqual(flac["prev"], prev)


class AddressRootNodetest(unittest.TestCase):
    def test_find_root_node(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conans.errors import ConanInvalidConfiguration

            class Pkg(ConanFile):
                def set_name(self):
                    self.name = "pkg"

                def set_version(self):
                    self.version = "0.1"
            """)
        client.save({"conanfile.py": conanfile})
        client.run("lock create conanfile.py --lockfile-out=conan.lock")
        client.run("install . --lockfile=conan.lock")
        self.assertIn("conanfile.py (pkg/0.1): Installing package", client.out)


def test_error_test_command():
    # https://github.com/conan-io/conan/issues/9088
    client = TestClient()
    client.save({"dep/conanfile.py": GenConanfile(),
                 "pkg/conanfile.py": GenConanfile().with_requires("dep/[>=1.0]"),
                 "test_package/conanfile.py": GenConanfile().with_test("pass")})
    client.run("create dep dep/1.0@")
    client.run("create pkg pkg/1.0@")
    client.run("lock create --ref=pkg/1.0@")
    client.run("create dep dep/1.1@")
    client.run("test test_package pkg/1.0@ --lockfile=conan.lock")
    assert "dep/1.0" in client.out
    assert "dep/1.1" not in client.out
    client.run("test test_package pkg/1.0@")
    assert "dep/1.0" not in client.out
    assert "dep/1.1" in client.out


def test_override_not_locked():
    # https://github.com/conan-io/conan/pull/8907
    client = TestClient()
    client.save({"dep/conanfile.py": GenConanfile(),
                 "pkg/conanfile.py": GenConanfile().with_requires("dep/[*]"),
                 "consumer/conanfile.py":
                     GenConanfile().with_requirement("pkg/1.0").with_requirement("dep/1.0",
                                                                                 override=True)})
    client.run("create dep dep/1.0@")
    client.run("create dep dep/1.1@")
    client.run("create pkg pkg/1.0@")
    client.run("lock create consumer/conanfile.py --lockfile-out=app1.lock")
    client.run("install consumer/conanfile.py --lockfile app1.lock")


def test_compatible_transient_options():
    # https://github.com/conan-io/conan/issues/9591
    client = TestClient()

    lib_base = GenConanfile().with_option("shared", [True, False])\
        .with_default_option("shared", False)

    lib_compatible = textwrap.dedent("""
        from conans import ConanFile
        class LibCompatibleConanFile(ConanFile):
            settings = "os"
            options = {"shared": [True, False]}
            default_options = {"shared": False}
            requires = "base/1.0"
            def package_id(self):
                if self.settings.os == "Windows":
                    compatible_pkg = self.info.clone()
                    compatible_pkg.settings.os = "Linux"
                    self.compatible_packages.append(compatible_pkg)
        """)
    consumer = GenConanfile().with_requires("compatible/1.0")
    client.save({"base/conanfile.py": lib_base,
                 "compat/conanfile.py": lib_compatible,
                 "consumer/conanfile.py": consumer})
    client.run("create base base/1.0@")
    client.run("create compat compatible/1.0@ -s os=Linux")
    client.run("lock create consumer/conanfile.py -s os=Windows --lockfile-out=deps.lock")
    client.run("install consumer/conanfile.py --lockfile=deps.lock")



import textwrap

from jinja2 import Template


def gen_cmakelists(language="CXX", verify=True, project="project", libname="mylibrary",
                   libsources=None, appname="myapp", appsources=None, cmake_version="3.15",
                   install=False, find_package=None, libtype="", deps=None, public_header=None,
                   custom_content=None):
    """
    language: C, C++, C/C++
    project: the project name
    """
    cmake = textwrap.dedent("""\
        {% if verify %}
        set(CMAKE_CXX_COMPILER_WORKS 1)
        set(CMAKE_CXX_ABI_COMPILED 1)
        set(CMAKE_C_COMPILER_WORKS 1)
        set(CMAKE_C_ABI_COMPILED 1)
        {% endif %}

        cmake_minimum_required(VERSION {{cmake_version}})
        project({{project}} {{language}})

        {% if find_package is mapping %}
        {% for s, c in find_package.items() %}
        find_package({{s}} COMPONENTS {{c}} )
        {% endfor %}
        {% else %}
        {% for s in find_package %}
        find_package({{s}})
        {% endfor %}
        {% endif %}

        {% if libsources %}
        add_library({{libname}} {{libtype}} {% for s in libsources %} {{s}} {% endfor %})
        target_include_directories({{libname}} PUBLIC "include")
        {% endif %}

        {% if libsources and find_package %}
        {% if find_package is mapping %}
        target_link_libraries({{libname}} {% for s, c in find_package.items() %} {{s}}::{{c}} {% endfor %})
        {% else %}
        target_link_libraries({{libname}} {% for s in find_package %} {{s}}::{{s}} {% endfor %})
        {% endif %}
        {% endif %}

        {% if libsources and deps %}
        target_link_libraries({{libname}} {% for s in deps %} {{s}} {% endfor %})
        {% endif %}

        {% if appsources %}
        add_executable({{appname}} {% for s in appsources %} {{s}} {% endfor %})
        target_include_directories({{appname}} PUBLIC "include")
        {% endif %}

        {% if appsources and libsources %}
        target_link_libraries({{appname}} {{libname}})
        {% endif %}

        {% if appsources and not libsources and find_package %}
        {% if find_package is mapping %}
         target_link_libraries({{appname}} {% for s, c in find_package.items() %} {{s}}::{{c}} {% endfor %})
        {% else %}
        target_link_libraries({{appname}} {% for s in find_package %} {{s}}::{{s}} {% endfor %})
        {% endif %}
        {% endif %}

        {% if appsources and deps %}
        target_link_libraries({{appname}} {% for s in deps %} {{s}} {% endfor %})
        {% endif %}

        {% if libsources and public_header %}
        set_target_properties({{libname}} PROPERTIES PUBLIC_HEADER "{{public_header}}")
        {% endif %}

        {% if install %}
        {% if appsources %}
        install(TARGETS {{appname}})
        {% endif %}
        {% if libsources %}
        install(TARGETS {{libname}})
        {% endif %}
        {% endif %}

        {% if custom_content %}
        {{custom_content}}
        {% endif %}

        """)

    t = Template(cmake, trim_blocks=True, lstrip_blocks=True)
    return t.render({"verify": verify,
                     "language": language,
                     "project": project,
                     "libname": libname,
                     "libsources": libsources,
                     "appname": appname,
                     "appsources": appsources,
                     "cmake_version": cmake_version,
                     "install": install,
                     "find_package": find_package or [],
                     "libtype": libtype,
                     "public_header": public_header,
                     "deps": deps,
                     "custom_content": custom_content
                     })



# -*- coding: utf-8 -*-

import unittest

from mock import mock

from conans.client.conf import get_default_settings_yml
from conans.client.tools.oss import cross_building
from conans.model.settings import Settings


# TODO: Add tests using a conanfile with 'settings' and 'settings_build'

class CrossBuildingTest(unittest.TestCase):
    def test_same(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "FreeBSD"
        settings.arch = "x86_64"
        settings.arch_build = "x86_64"
        self.assertFalse(cross_building(settings, self_os="FreeBSD", self_arch="x86_64"))

        with mock.patch("platform.system", mock.MagicMock(return_value='FreeBSD')),\
             mock.patch("platform.machine", mock.MagicMock(return_value="x86_64")):
            self.assertFalse(cross_building(settings))

        settings.os_build = "FreeBSD"
        settings.arch = "x86_64"
        self.assertFalse(cross_building(settings))

    def test_different_os(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Linux"
        settings.arch = "x86_64"
        self.assertTrue(cross_building(settings, self_os="FreeBSD", self_arch="x86_64"))

        with mock.patch("platform.system", mock.MagicMock(return_value='FreeBSD')),\
             mock.patch("platform.machine", mock.MagicMock(return_value="x86_64")):
            self.assertTrue(cross_building(settings))

        settings.os_build = "FreeBSD"
        settings.arch_build = "x86_64"
        self.assertTrue(cross_building(settings))

    def test_different_arch(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "FreeBSD"
        settings.arch = "x86"
        self.assertTrue(cross_building(settings, self_os="FreeBSD", self_arch="x86_64"))

        with mock.patch("platform.system", mock.MagicMock(return_value='FreeBSD')), \
             mock.patch("platform.machine", mock.MagicMock(return_value="x86_64")):
            self.assertTrue(cross_building(settings))

        settings.os_build = "FreeBSD"
        settings.arch_build = "x86_64"
        self.assertTrue(cross_building(settings))

    def test_x64_x86(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "FreeBSD"
        settings.arch = "x86"
        self.assertFalse(cross_building(settings,  self_os="FreeBSD",
                                        self_arch="x86_64", skip_x64_x86=True))
        self.assertTrue(cross_building(settings, self_os="FreeBSD",
                                       self_arch="x86_64", skip_x64_x86=False))

        with mock.patch("platform.system", mock.MagicMock(return_value='FreeBSD')), \
             mock.patch("platform.machine", mock.MagicMock(return_value="x86_64")):
            self.assertFalse(cross_building(settings, skip_x64_x86=True))
            self.assertTrue(cross_building(settings, skip_x64_x86=False))

        settings.os_build = "FreeBSD"
        settings.arch_build = "x86_64"
        self.assertFalse(cross_building(settings, skip_x64_x86=True))
        self.assertTrue(cross_building(settings, skip_x64_x86=False))

    def test_x86_x64(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "FreeBSD"
        settings.arch = "x86_64"
        self.assertTrue(cross_building(settings,  self_os="FreeBSD",
                                       self_arch="x86", skip_x64_x86=True))
        self.assertTrue(cross_building(settings, self_os="FreeBSD",
                                       self_arch="x86", skip_x64_x86=False))

        with mock.patch("platform.system", mock.MagicMock(return_value='FreeBSD')), \
             mock.patch("platform.machine", mock.MagicMock(return_value="x86")):
            self.assertTrue(cross_building(settings, skip_x64_x86=True))
            self.assertTrue(cross_building(settings, skip_x64_x86=False))

        settings.os_build = "FreeBSD"
        settings.arch_build = "x86"
        self.assertTrue(cross_building(settings, skip_x64_x86=True))
        self.assertTrue(cross_building(settings, skip_x64_x86=False))

    def test_x86_64_different_os(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Linux"
        settings.arch = "x86"
        self.assertTrue(cross_building(settings,  self_os="FreeBSD",
                                       self_arch="x86_64", skip_x64_x86=True))
        self.assertTrue(cross_building(settings, self_os="FreeBSD",
                                       self_arch="x86_64", skip_x64_x86=False))

        with mock.patch("platform.system", mock.MagicMock(return_value='FreeBSD')), \
             mock.patch("platform.machine", mock.MagicMock(return_value="x86_64")):
            self.assertTrue(cross_building(settings, skip_x64_x86=True))
            self.assertTrue(cross_building(settings, skip_x64_x86=False))

        settings.os_build = "FreeBSD"
        settings.arch_build = "x86_64"
        self.assertTrue(cross_building(settings, skip_x64_x86=True))
        self.assertTrue(cross_building(settings, skip_x64_x86=False))



import os
import shutil

from conans.client.file_copier import FileCopier
from conans.client.output import ScopedOutput
from conans.client.packager import report_files_from_manifest
from conans.errors import ConanException, conanfile_exception_formatter
from conans.model.conan_file import get_env_context_manager
from conans.model.manifest import FileTreeManifest
from conans.paths import CONANINFO
from conans.tools import chdir, mkdir
from conans.util.conan_v2_mode import conan_v2_property
from conans.util.files import save, mkdir
from conans.util.log import logger


def run_deb_method(conanfile, deb_id, hook_manager, conanfile_path, ref, copy_info=False):
    """ calls the recipe "deb()" method
    - Assigns folders to conanfile.deb_folder, source_folder, install_folder, build_folder
    - Calls pre-post deb hook
    - Prepares FileCopier helper for self.copy
    """

    if conanfile.deb_folder == conanfile.build_folder:
        raise ConanException("Cannot 'conan deb' to the build folder. "
                             "--build-folder and deb folder can't be the same")

    mkdir(conanfile.deb_folder)
    output = conanfile.output
    # Make the copy of all the patterns
    output.info("Generating the deb")
    output.info("deb folder %s" % conanfile.deb_folder)

    with get_env_context_manager(conanfile):
        return _call_deb(conanfile, deb_id, hook_manager, conanfile_path, ref, copy_info)


def _call_deb(conanfile, deb_id, hook_manager, conanfile_path, ref, copy_info):
    output = conanfile.output

    # hook_manager.execute("pre_deb", conanfile=conanfile, conanfile_path=conanfile_path,
    #                      reference=ref, deb_id=deb_id)

    output.highlight("Calling deb()")
    folders = [conanfile.source_folder, conanfile.package_folder] \
        if conanfile.source_folder != conanfile.package_folder else [conanfile.package_folder]
    conanfile.copy = FileCopier(folders, conanfile.deb_folder)
    with conanfile_exception_formatter(str(conanfile), "deb"):
        with chdir(conanfile.package_folder):
            with conan_v2_property(conanfile, 'info',
                                   "'self.info' access in deb() method is deprecated"):
                _create_deb_file(conanfile)
                conanfile.deb()

    # hook_manager.execute("post_deb", conanfile=conanfile, conanfile_path=conanfile_path,
    #                      reference=ref, deb_id=deb_id)

    manifest = _create_aux_files(conanfile, copy_info)
    deb_output = ScopedOutput("%s deb()" % output.scope, output)
    report_files_from_manifest(deb_output, manifest)
    deb_id = deb_id or os.path.basename(conanfile.deb_folder)

    output.success("Deb '%s' created" % deb_id)

    prev = manifest.summary_hash
    output.info("Created deb revision %s" % prev)
    return prev


def _create_aux_files(conanfile, copy_info):
    """ auxiliary method that creates CONANINFO and manifest in
    the deb_folder
    """
    logger.debug("DEB: Creating config files to %s" % conanfile.deb_folder)
    if copy_info:
        try:
            shutil.copy(os.path.join(conanfile.install_folder, CONANINFO),
                        conanfile.deb_folder)
        except IOError:
            raise ConanException("%s does not exist inside of your %s folder. "
                                 "Try to re-build it again to solve it."
                                 % (CONANINFO, conanfile.install_folder))
    else:
        save(os.path.join(conanfile.deb_folder, CONANINFO), conanfile.info.dumps())

    # Create the digest for the deb
    manifest = FileTreeManifest.create(conanfile.deb_folder)
    manifest.save(conanfile.deb_folder)
    return manifest

def _create_deb_file(conanfile):
    output = conanfile.output
    #### set folder name
    package_name = conanfile.name
    package_version = conanfile.version
    deb_folder = package_name
    deb_file = package_name + "_" + package_version + ".deb"
    debian_folder = deb_folder + "/DEBIAN"
    app_folder = deb_folder + "/sense2"
        
    #### generate package in build
    mkdir(debian_folder)
    mkdir(app_folder)
        
    ## add changelog file
    changelog_file = debian_folder + "/changelog"
    if os.path.exists(changelog_file):
        conanfile.run("rm " + changelog_file)
    add_changelog_cmd = "echo " + conanfile.changelog + " >> " + changelog_file
    conanfile.run(add_changelog_cmd)
    
    ## add control file
    control_file = debian_folder + "/control"
    if os.path.exists(control_file):
        conanfile.run("rm " + control_file)
    add_control_cmd = "echo Package: " + conanfile.deb_control["Package"] + " >> " + control_file + " && " \
                        "echo Version: " + conanfile.deb_control["Version"] + " >> " + control_file + " && "  \
                        "echo Maintainer: " + conanfile.deb_control["Maintainer"] + " >> " + control_file + " && "  \
                        "echo Architecture: " + conanfile.deb_control["Architecture"] + " >> " + control_file + " && "  \
                        "echo Description: " + conanfile.deb_control["Description"] + " >> " + control_file + " && "  \
                        "echo 'Depends: " + conanfile.deb_control["Depends"] + "' >> " + control_file + " && "  \
                        "echo Essential: " + conanfile.deb_control["Essential"] + " >> " + control_file + " "
    conanfile.run(add_control_cmd)  
        
    ## add lib | bin
    add_lib_cmd = "cp -r lib " + app_folder + "/lib "
    if os.path.exists("lib"):
        conanfile.run(add_lib_cmd)
    
    add_bin_cmd = "cp -r bin " + app_folder + "/bin "
    if os.path.exists("bin"):
        conanfile.run(add_bin_cmd)

    ## generate deb package in build
    build_package_cmd = "dpkg-deb --build " + deb_folder + " " + deb_file
    conanfile.run(build_package_cmd)

    ## copy *.deb in package folder to deb folder
    conanfile.copy(deb_file, dst=".", keep_path=False)
    output.success("copy '%s' to deb folder" % deb_file)
    
    ## clean package folder
    if os.path.exists(deb_folder):
            conanfile.run("rm -rf " + deb_folder)

    if os.path.exists(deb_file):
        conanfile.run("rm -rf " + deb_file)
    


import textwrap
import unittest

from conans.client.remote_manager import CONAN_REQUEST_HEADER_SETTINGS, CONAN_REQUEST_HEADER_OPTIONS
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient, TestServer, TestRequester
from conans.util.env_reader import get_env


class RequesterClass(TestRequester):
    requests = None

    def __init__(self, *args, **kwargs):
        self.requests = []
        super(RequesterClass, self).__init__(*args, **kwargs)

    def get(self, url, headers=None, **kwargs):
        self.requests.append((url, headers))
        return super(RequesterClass, self).get(url, headers=headers, **kwargs)


class RequestHeadersTestCase(unittest.TestCase):
    """ Conan adds a header with the settings used to compute the package ID """
    revs_enabled = get_env("TESTING_REVISIONS_ENABLED", False)

    profile = textwrap.dedent("""
        [settings]
        os=Macos
        arch=x86_64
        compiler=apple-clang
        compiler.version=11.0
        compiler.libcxx=libc++
        build_type=Release
    """)

    conanfile = GenConanfile().with_settings('os', 'arch', 'compiler') \
        .with_option('opt1', [True, False]) \
        .with_option('shared', [True, False]) \
        .with_default_option('opt1', True) \
        .with_default_option('shared', False)

    def setUp(self):
        test_server = TestServer(users={"user": "mypass"})
        self.servers = {"default": test_server}
        t = TestClient(servers=self.servers, users={"default": [("user", "mypass")]})
        t.save({'conanfile.py': self.conanfile,
                'profile': self.profile})
        t.run('create conanfile.py name/version@user/channel --profile:host=profile')
        t.run('upload name/version@user/channel --all')

    def _get_header(self, requester, header_name):
        hits = sum([header_name in headers for _, headers in requester.requests])
        assert hits <= 2 if self.revs_enabled else 1
        for url, headers in requester.requests:
            if header_name in headers:
                if self.revs_enabled:
                    self.assertTrue(url.endswith('/latest'), msg=url)
                else:
                    self.assertTrue(url.endswith('/download_urls'), msg=url)
                return headers.get(header_name)

    def _assert_settings_headers(self, settings_header, compiler_version='11.0'):
        # It takes only the values that are relevant to the recipe
        self.assertListEqual(
            sorted(['os', 'arch', 'compiler', 'compiler.version', 'compiler.libcxx']),
            sorted([it.split('=', 1)[0] for it in settings_header.split(';')]))
        self.assertIn('os=Macos', settings_header)
        self.assertIn('arch=x86_64', settings_header)
        self.assertIn('compiler=apple-clang', settings_header)
        self.assertIn('compiler.libcxx=libc++', settings_header)
        self.assertIn('compiler.version={}'.format(compiler_version), settings_header)
        self.assertNotIn('build_type', settings_header)

    def _assert_options_headers(self, options_header, shared_value='False'):
        self.assertListEqual(['shared'], [it.split('=', 1)[0] for it in options_header.split(';')])
        self.assertIn('shared={}'.format(shared_value), options_header)

    def _get_test_client(self):
        t = TestClient(requester_class=RequesterClass, servers=self.servers,
                       users={"default": [("user", "mypass")]})
        t.run('config set general.revisions_enabled={}'.format('1' if self.revs_enabled else '0'))
        return t

    def test_install_recipe_mismatch(self):
        t = self._get_test_client()
        t.save({'profile': self.profile})
        t.run('install failing/version@user/channel --profile=profile', assert_error=True)
        self.assertFalse(any([CONAN_REQUEST_HEADER_SETTINGS in headers for _, headers in
                              t.api.http_requester.requests]))
        self.assertFalse(any([CONAN_REQUEST_HEADER_OPTIONS in headers for _, headers in
                              t.api.http_requester.requests]))

    def test_install_package_match(self):
        t = self._get_test_client()
        t.save({'profile': self.profile})

        # Package match
        t.run('install name/version@user/channel --profile=profile')
        settings_header = self._get_header(t.api.http_requester, CONAN_REQUEST_HEADER_SETTINGS)
        self._assert_settings_headers(settings_header)
        options_headers = self._get_header(t.api.http_requester, CONAN_REQUEST_HEADER_OPTIONS)
        self._assert_options_headers(options_headers)

        # Package mismatch (settings)
        t.run('install name/version@user/channel --profile=profile -s compiler.version=12.0',
              assert_error=True)
        settings_header = self._get_header(t.api.http_requester, CONAN_REQUEST_HEADER_SETTINGS)
        self._assert_settings_headers(settings_header, compiler_version='12.0')

        # Package mismatch (options)
        t.run('install name/version@user/channel --profile=profile -o shared=True',
              assert_error=True)
        options_headers = self._get_header(t.api.http_requester, CONAN_REQUEST_HEADER_OPTIONS)
        self._assert_options_headers(options_headers, shared_value='True')

    def test_info_package_match(self):
        t = self._get_test_client()
        t.save({'profile': self.profile})

        # Package match
        t.run('info name/version@user/channel --profile=profile')
        settings_header = self._get_header(t.api.http_requester, CONAN_REQUEST_HEADER_SETTINGS)
        self._assert_settings_headers(settings_header)
        options_headers = self._get_header(t.api.http_requester, CONAN_REQUEST_HEADER_OPTIONS)
        self._assert_options_headers(options_headers)

        # Package mismatch (settings)
        t.run('info name/version@user/channel --profile=profile -s compiler.version=12.0')
        settings_header = self._get_header(t.api.http_requester, CONAN_REQUEST_HEADER_SETTINGS)
        self._assert_settings_headers(settings_header, compiler_version='12.0')

        # Package mismatch (options)
        t.run('install name/version@user/channel --profile=profile -o shared=True',
              assert_error=True)
        options_headers = self._get_header(t.api.http_requester, CONAN_REQUEST_HEADER_OPTIONS)
        self._assert_options_headers(options_headers, shared_value='True')

    def test_install_as_requirement(self):
        t = self._get_test_client()
        t.save({'conanfile.py': GenConanfile().with_requires('name/version@user/channel'),
                'profile': self.profile})

        # Requirement is found
        t.run('install . consumer/version@ --profile=profile')
        settings_header = self._get_header(t.api.http_requester, CONAN_REQUEST_HEADER_SETTINGS)
        self._assert_settings_headers(settings_header)
        options_headers = self._get_header(t.api.http_requester, CONAN_REQUEST_HEADER_OPTIONS)
        self._assert_options_headers(options_headers)

        # Requirement is not found (settings)
        t.run('install . consumer/version@ --profile=profile -s compiler.version=12.0',
              assert_error=True)
        settings_header = self._get_header(t.api.http_requester, CONAN_REQUEST_HEADER_SETTINGS)
        self._assert_settings_headers(settings_header, compiler_version='12.0')

        # Requirement is not found (options)
        t.run('install . consumer/version@ --profile=profile -o name:shared=True', assert_error=True)
        options_headers = self._get_header(t.api.http_requester, CONAN_REQUEST_HEADER_OPTIONS)
        self._assert_options_headers(options_headers, shared_value='True')



import platform
import re
import os
import shutil
import textwrap

import pytest

from conans.model.ref import ConanFileReference, PackageReference
from conans.test.utils.tools import TestClient, TestServer


@pytest.mark.skipif(platform.system() not in ["Linux", "Darwin"], reason="Requires Autotools")
@pytest.mark.tool_autotools()
def test_autotools_lib_template():
    client = TestClient(path_with_spaces=False)
    client.run("new hello/0.1 --template=autotools_lib")

    # Local flow works
    client.run("install . -if=install")
    client.run("build . -if=install")

    client.run("export-pkg . hello/0.1@ -if=install")
    package_id = re.search(r"Packaging to (\S+)", str(client.out)).group(1)
    pref = PackageReference(ConanFileReference.loads("hello/0.1"), package_id)
    package_folder = client.cache.package_layout(pref.ref).package(pref)
    assert os.path.exists(os.path.join(package_folder, "include", "hello.h"))

    # Create works
    client.run("create .")
    assert "hello/0.1: Hello World Release!" in client.out

    client.run("create . -s build_type=Debug")
    assert "hello/0.1: Hello World Debug!" in client.out

    # Create + shared works
    client.save({}, clean_first=True)
    client.run("new hello/0.1 --template=autotools_lib")
    client.run("create . -o hello:shared=True")
    assert "hello/0.1: Hello World Release!" in client.out
    if platform.system() == "Darwin":
        client.run_command("otool -l test_package/build-release/main")
        assert "libhello.0.dylib" in client.out
    else:
        client.run_command("ldd test_package/build-release/main")
        assert "libhello.so.0" in client.out


@pytest.mark.skipif(platform.system() not in ["Linux", "Darwin"], reason="Requires Autotools")
@pytest.mark.tool_autotools()
def test_autotools_exe_template():
    client = TestClient(path_with_spaces=False)
    client.run("new greet/0.1 --template=autotools_exe")
    # Local flow works
    client.run("install . -if=install")
    client.run("build . -if=install")

    # Create works
    client.run("create .")
    # check that for exe's we don't add any static/shared flag
    for flag in ["--enable-static", "--disable-static", "--disable-shared", "--with-pic"]:
        assert flag not in client.out
    assert "greet/0.1: Hello World Release!" in client.out

    client.run("create . -s build_type=Debug")
    for flag in ["--enable-static", "--disable-static", "--disable-shared", "--with-pic"]:
        assert flag not in client.out
    assert "greet/0.1: Hello World Debug!" in client.out


@pytest.mark.skipif(platform.system() not in ["Darwin"], reason="Requires Autotools")
@pytest.mark.tool_autotools()
def test_autotools_relocatable_libs_darwin():
    client = TestClient(path_with_spaces=False)
    client.run("new hello/0.1 --template=autotools_lib")
    client.run("create . -o hello:shared=True")
    package_id = re.search(r"Package (\S+)", str(client.out)).group(1)
    package_id = package_id.replace("'", "")
    pref = PackageReference(ConanFileReference.loads("hello/0.1"), package_id)
    package_folder = client.cache.package_layout(pref.ref).package(pref)
    dylib = os.path.join(package_folder, "lib", "libhello.0.dylib")
    if platform.system() == "Darwin":
        client.run_command("otool -l {}".format(dylib))
        assert "@rpath/libhello.0.dylib" in client.out
        client.run_command("otool -l {}".format("test_package/build-release/main"))

    # will work because rpath set
    client.run_command("test_package/build-release/main")
    assert "hello/0.1: Hello World Release!" in client.out

    # move to another location so that the path set in the rpath does not exist
    # then the execution should fail
    shutil.move(os.path.join(package_folder, "lib"), os.path.join(client.current_folder, "tempfolder"))
    # will fail because rpath does not exist
    client.run_command("test_package/build-release/main", assert_error=True)
    assert "Library not loaded: @rpath/libhello.0.dylib" in str(client.out).replace("'", "")

    # Use DYLD_LIBRARY_PATH and should run
    client.run_command("DYLD_LIBRARY_PATH={} test_package/build-release/main".format(os.path.join(client.current_folder, "tempfolder")))
    assert "hello/0.1: Hello World Release!" in client.out


@pytest.mark.skipif(platform.system() not in ["Darwin"], reason="Requires Autotools")
@pytest.mark.tool_autotools()
def test_autotools_relocatable_libs_darwin_downloaded():
    server = TestServer([("*/*@*/*", "*")], [("*/*@*/*", "*")])
    client = TestClient(servers={"default": server}, path_with_spaces=False)
    client2 = TestClient(servers={"default": server}, path_with_spaces=False)
    assert client2.cache_folder != client.cache_folder
    client.run("new hello/0.1 --template=autotools_lib")
    client.run("create . -o hello:shared=True -tf=None")
    client.run("upload * --all -c")
    client.run("remove * -f")

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.gnu import Autotools
        from conan.tools.layout import basic_layout

        class GreetConan(ConanFile):
            name = "greet"
            version = "1.0"
            settings = "os", "compiler", "build_type", "arch"
            exports_sources = "configure.ac", "Makefile.am", "main.cpp"
            generators = "AutotoolsDeps", "AutotoolsToolchain"

            def requirements(self):
                self.requires("hello/0.1")

            def layout(self):
                basic_layout(self)

            def build(self):
                autotools = Autotools(self)
                autotools.autoreconf()
                autotools.configure()
                autotools.make()
        """)

    main = textwrap.dedent("""
        #include "hello.h"
        int main() { hello(); }
        """)

    makefileam = textwrap.dedent("""
        bin_PROGRAMS = greet
        greet_SOURCES = main.cpp
        """)

    configureac = textwrap.dedent("""
        AC_INIT([greet], [1.0], [])
        AM_INIT_AUTOMAKE([-Wall -Werror foreign])
        AC_PROG_CXX
        AM_PROG_AR
        LT_INIT
        AC_CONFIG_FILES([Makefile])
        AC_OUTPUT
        """)

    client2.save({"conanfile.py": conanfile,
                  "main.cpp": main,
                  "makefile.am": makefileam,
                  "configure.ac": configureac})

    client2.run("install . -o hello:shared=True -r default")
    client2.run("build .")
    client2.run_command("build-release/greet")
    assert "Hello World Release!" in client2.out


@pytest.mark.skipif(platform.system() not in ["Darwin"], reason="Only affects apple platforms")
@pytest.mark.tool_autotools()
def test_autotools_fix_shared_libs():
    """
    From comments in: https://github.com/conan-io/conan/pull/11365

    Case 1:
    libopencv_core.3.4.17.dylib
    libopencv_core.3.4.dylib (symlink) -> libopencv_core.3.4.17.dylib
    libopencv_core.dylib (symlink) -> libopencv_core.3.4.dylib

    Install name in libopencv_core.3.4.17.dylib is libopencv_core.3.4.dylib NOT the dylib name
    So we have to add the rpath to that.

    Case 2:
    libopencv_core.dylib
    libopencv_imgproc.dylib

    libopencv_imgproc.dylib depends on libopencv_core.dylib and declares that dependency not using the
    @rpath, we have to make sure that we patch the dependencies in the dylibs using install_name_tool -change

    Let's create a Conan package with two libraries: bye and hello (bye depends on hello)
    and recreate this whole situation to check that we are correctly fixing the dylibs
    """
    client = TestClient(path_with_spaces=False)
    client.run("new hello/0.1 --template=autotools_lib")

    conanfile = textwrap.dedent("""
        import os

        from conan import ConanFile
        from conan.tools.gnu import AutotoolsToolchain, Autotools
        from conan.tools.layout import basic_layout
        from conan.tools.apple import fix_apple_shared_install_name


        class HelloConan(ConanFile):
            name = "hello"
            version = "0.1"
            settings = "os", "compiler", "build_type", "arch"
            options = {"shared": [True, False], "fPIC": [True, False]}
            default_options = {"shared": False, "fPIC": True}
            exports_sources = "configure.ac", "Makefile.am", "src/*"

            def layout(self):
                basic_layout(self)

            def generate(self):
                at_toolchain = AutotoolsToolchain(self)
                at_toolchain.generate()

            def build(self):
                autotools = Autotools(self)
                autotools.autoreconf()
                autotools.configure()
                autotools.make()

            def package(self):
                autotools = Autotools(self)
                autotools.install()
                # before fixing the names we try to reproduce the two cases explained
                # in the test that dylib name and install name are not the same
                self.run("install_name_tool {} -id /lib/libbye.dylib".format(os.path.join(self.package_folder,
                                                                                          "lib", "libbye.0.dylib")))
                # also change that in the libbye dependencies
                self.run("install_name_tool {} -change /lib/libhello.0.dylib /lib/libhello.dylib".format(os.path.join(self.package_folder,
                                                                                                         "lib", "libbye.0.dylib")))
                self.run("install_name_tool {} -id /lib/libhello.dylib".format(os.path.join(self.package_folder,
                                                                                            "lib","libhello.0.dylib")))
                fix_apple_shared_install_name(self)

            def package_info(self):
                self.cpp_info.libs = ["hello", "bye"]
    """)

    bye_cpp = textwrap.dedent("""
        #include <iostream>
        #include "hello.h"
        #include "bye.h"
        void bye(){
            hello();
            std::cout << "Bye, bye!" << std::endl;
        }
    """)

    bye_h = textwrap.dedent("""
        #pragma once
        void bye();
    """)

    makefile_am = textwrap.dedent("""
        lib_LTLIBRARIES = libhello.la libbye.la

        libhello_la_SOURCES = hello.cpp hello.h
        libhello_la_HEADERS = hello.h
        libhello_ladir = $(includedir)

        libbye_la_SOURCES = bye.cpp bye.h
        libbye_la_HEADERS = bye.h
        libbye_ladir = $(includedir)
        libbye_la_LIBADD = libhello.la
    """)

    test_src = textwrap.dedent("""
        #include "bye.h"
        int main() { bye(); }
    """)

    client.save({
        "src/makefile.am": makefile_am,
        "src/bye.cpp": bye_cpp,
        "src/bye.h": bye_h,
        "test_package/main.cpp": test_src,
        "conanfile.py": conanfile,
    })

    client.run("create . -o hello:shared=True -tf=None")

    package_id = re.search(r"Package (\S+)", str(client.out)).group(1)
    package_id = package_id.replace("'", "")
    pref = PackageReference(ConanFileReference.loads("hello/0.1"), package_id)
    package_folder = client.cache.package_layout(pref.ref).package(pref)

    # install name fixed
    client.run_command("otool -D {}".format(os.path.join(package_folder, "lib", "libhello.0.dylib")))
    assert "@rpath/libhello.dylib" in client.out
    client.run_command("otool -D {}".format(os.path.join(package_folder, "lib", "libbye.0.dylib")))
    assert "@rpath/libbye.dylib" in client.out

    # dependencies fixed
    client.run_command("otool -L {}".format(os.path.join(package_folder, "lib", "libbye.0.dylib")))
    assert "/lib/libhello.dylib (compatibility version 1.0.0, current version 1.0.0)" not in client.out
    assert "/lib/libbye.dylib (compatibility version 1.0.0, current version 1.0.0)" not in client.out
    assert "@rpath/libhello.dylib (compatibility version 1.0.0, current version 1.0.0)" in client.out
    assert "@rpath/libbye.dylib (compatibility version 1.0.0, current version 1.0.0)" in client.out

    client.run("test test_package hello/0.1@ -o hello:shared=True")
    assert "Bye, bye!" in client.out
